import importlib
import subprocess
import pkg_resources
import glob
import os
import inspect


def get_provider_options(provider_name):
    module = importlib.import_module(f"provider.{provider_name}")
    provider_class = getattr(module, f"{provider_name.capitalize()}Provider")
    signature = inspect.signature(provider_class.__init__)
    options = [
        param for param in signature.parameters if param != "self" and param != "kwargs"
    ]
    return options


class Provider:
    def __init__(self, name, **kwargs):
        try:
            module = importlib.import_module(f"provider.{name}")
            provider_class = getattr(module, f"{name.capitalize()}Provider")
            self.instance = provider_class(**kwargs)

            # Install the requirements if any
            self.install_requirements()

        except (ModuleNotFoundError, AttributeError) as e:
            raise AttributeError(f"module {__name__} has no attribute {name}") from e

    def __getattr__(self, attr):
        return getattr(self.instance, attr)

    def get_providers(self):
        providers = []
        for provider in glob.glob("provider/*.py"):
            if "__init__.py" not in provider:
                providers.append(os.path.splitext(os.path.basename(provider))[0])
        return providers

    def install_requirements(self):
        requirements = getattr(self.instance, "requirements", [])
        installed_packages = {pkg.key: pkg.version for pkg in pkg_resources.working_set}
        for requirement in requirements:
            if requirement.lower() not in installed_packages:
                subprocess.run(["pip", "install", requirement], check=True)


def __getattr__(name):
    return Provider(name)


def max_tokens_ceiling(ai_model: str):
    """Generates the max token limit for a given model"""

    # https://huggingface.co/OpenAssistant/oasst-sft-6-llama-30b-xor
    if ai_model == "openassistant":
        return 2000
    # https://huggingface.co/bigcode/starcoderbase
    elif ai_model == "starcoderbase":
        return 8192
    elif ai_model == "default":
        return 2000
    else:
        return 999999999



from Extensions import Extensions
from Config import Config
from Chain import Chain
from AGiXT import AGiXT
import json
import os
from typing import List, Optional


class agixt_agent(Extensions):
    def __init__(self, **kwargs):
        agents = Config().get_agents()
        self.chains = Chain().get_chains()
        self.commands = {
            "Evaluate Code": self.evaluate_code,
            "Analyze Pull Request": self.analyze_pull_request,
            "Perform Automated Testing": self.perform_automated_testing,
            "Run CI-CD Pipeline": self.run_ci_cd_pipeline,
            "Improve Code": self.improve_code,
            "Write Tests": self.write_tests,
            "Create a new command": self.create_command,
        }
        if agents != None:
            for agent in agents:
                if "name" in agent:
                    name = f" AI Agent {agent['name']}"
                    self.commands.update(
                        {f"Ask{name}": self.ask, f"Instruct{name}": self.instruct}
                    )
        if self.chains != None:
            for chain in self.chains:
                if "name" in chain:
                    self.commands.update(
                        {f"Run Chain: {chain['name']}": self.run_chain}
                    )

    def command_exists(self, file_name: str) -> bool:
        return os.path.exists(f"commands/{file_name}.py")

    async def create_command(
        self, function_description: str, agent: str = "AGiXT"
    ) -> List[str]:
        with open(f"prompts/Create New Command.txt", "r") as f:
            prompt = f.read()
        prompt = prompt.replace("{{NEW_FUNCTION_DESCRIPTION}}", function_description)
        response = await AGiXT(agent).run(prompt)
        file_name = response.split("class ")[1].split("(")[0]
        code = code.replace("```", "")

        if not self.command_exists(file_name):
            with open(f"commands/{file_name}.py", "w") as f:
                f.write(code)
            return f"Created new command: {file_name}."
        else:
            return f"Command {file_name} already exists. No changes were made."

    async def evaluate_code(self, code: str, agent: str = "AGiXT") -> List[str]:
        args = [code]
        function_string = "def analyze_code(code: str) -> List[str]:"
        description_string = "Analyzes the given code and returns a list of suggestions for improvements."
        prompt = f"You are now the following python function: ```# {description_string}\n{function_string}```\n\nOnly respond with your `return` value. Args: {args}"
        return await AGiXT(agent).run(prompt)

    async def analyze_pull_request(
        self, pr_url: str, agent: str = "AGiXT"
    ) -> List[str]:
        args = [pr_url]
        function_string = "def analyze_pr(pr_url: str) -> List[str]:"
        description_string = "Analyzes the given pull request and returns a list of suggestions for improvements."
        prompt = f"You are now the following python function: ```# {description_string}\n{function_string}```\n\nOnly respond with your `return` value. Args: {args}"
        return await AGiXT(agent).run(prompt)

    async def perform_automated_testing(
        self, test_url: str, agent: str = "AGiXT"
    ) -> List[str]:
        args = [test_url]
        function_string = "def perform_testing(test_url: str) -> List[str]:"
        description_string = "Performs automated testing using AI-driven tools and returns a list of test results."
        prompt = f"You are now the following python function: ```# {description_string}\n{function_string}```\n\nOnly respond with your `return` value. Args: {args}"
        return await AGiXT(agent).run(prompt)

    async def improve_code(
        self, suggestions: List[str], code: str, agent: str = "AGiXT"
    ) -> str:
        args = [json.dumps(suggestions), code]
        function_string = (
            "def generate_improved_code(suggestions: List[str], code: str) -> str:"
        )
        description_string = "Improves the provided code based on the suggestions provided, making no other changes."
        prompt = f"You are now the following python function: ```# {description_string}\n{function_string}```\n\nOnly respond with your `return` value. Args: {args}"
        return await AGiXT(agent).run(prompt)

    async def write_tests(
        self,
        code: str,
        focus: Optional[List[str]] = None,
        agent: str = "AGiXT",
    ) -> str:
        args = [code, json.dumps(focus) if focus else "None"]
        function_string = "def create_test_cases(code: str, focus: Optional[List[str]] = None) -> str:"
        description_string = "Generates test cases for the existing code, focusing on specific areas if required."
        prompt = f"You are now the following python function: ```# {description_string}\n{function_string}```\n\nOnly respond with your `return` value. Args: {args}"
        return await AGiXT(agent).run(prompt)

    async def run_ci_cd_pipeline(self, repo_url: str, agent: str = "AGiXT") -> str:
        args = [repo_url]
        function_string = "def run_pipeline(repo_url: str) -> str:"
        description_string = (
            "Runs the entire CI/CD pipeline for the given repository URL."
        )
        prompt = f"You are now the following python function: ```# {description_string}\n{function_string}```\n\nOnly respond with your `return` value. Args: {args}"
        return await AGiXT(agent).run(prompt)

    async def run_chain(self, chain_name):
        await Chain().run_chain(chain_name)
        return "Chain started successfully."

    async def ask(self, prompt: str, agent: str = "AGiXT") -> str:
        response = await AGiXT(agent).run(
            prompt, prompt="chat", websearch=True, websearch_depth=4
        )
        return response

    async def instruct(self, prompt: str, agent: str = "AGiXT") -> str:
        response = await AGiXT(agent).run(
            task=prompt, prompt="instruct", websearch=True, websearch_depth=8
        )
        return response



import streamlit as st
from Tasks import Tasks
from auth_libs.Users import check_auth_status
from pathlib import Path
from components.agent_selector import agent_selector
import asyncio

check_auth_status()

agent_name, agent = agent_selector()
st.title("Manage Tasks")


if agent_name:
    smart_task_toggle = st.checkbox("Enable Smart Task")
    task_objective = st.text_area("Enter the task objective")
    task_agent = Tasks(agent_name)
    task_list_dir = Path(f"agents/{agent_name}")
    task_list_dir.mkdir(parents=True, exist_ok=True)
    existing_tasks = task_agent.get_tasks_files()
    status = task_agent.get_status()
    agent_status = "Not Running" if status == False else "Running"
    load_task = st.selectbox(
        "Load Task",
        options=[""] + existing_tasks,
        index=0,
    )

    col1, col2 = st.columns([3, 1])
    with col1:
        columns = st.columns([3, 2])
        if st.button("Start Task", key=f"start_{agent_name}"):
            if agent_name and (task_objective or load_task):
                asyncio.run(
                    task_agent.run_task(
                        objective=task_objective,
                        smart=smart_task_toggle,
                        load_task=load_task,
                    )
                )
                st.experimental_rerun()
            else:
                columns[0].error("Agent name and task objective are required.")

        if st.button("Stop Task", key=f"stop_{agent_name}"):
            task_agent.stop_tasks()
            st.experimental_rerun()

    with col2:
        st.markdown(f"**Status:** {agent_status}")



# The Agent class is a Python class that represents an AI agent and provides methods for interacting
# with it, managing its configuration and memory, and executing commands.
# The Agent class is a Python class that represents an AI agent and provides methods for interacting
# with it, managing its configuration and memory, and executing commands.
import os
import json
import glob
import shutil
import importlib
import yaml
import time
from pathlib import Path
from inspect import signature, Parameter
from provider import Provider
from Memories import Memories
from Extensions import Extensions

DEFAULT_SETTINGS = {
    "provider": "gpt4free",
    "AI_MODEL": "gpt-4",
    "AI_TEMPERATURE": "0.7",
    "MAX_TOKENS": "4000",
    "embedder": "default",
    "LOG_REQUESTS": False,
}


class Agent:
    def __init__(self, agent_name=None):
        """
        This is the initialization function for an agent, which loads configurations and settings for the
        agent, including AI-related settings.

        :param agent_name: The name of the agent being initialized. If no name is provided, it defaults to
        "AGiXT"
        """
        # General Configuration
        self.agent_name = agent_name if agent_name is not None else "AGiXT"
        # Need to get the following from the agent config file:
        self.AGENT_CONFIG = self.get_agent_config()
        self.commands = self.load_commands()
        self.available_commands = Extensions(self.AGENT_CONFIG).get_available_commands()
        self.clean_agent_config_commands()
        # AI Configuration
        if "settings" in self.AGENT_CONFIG:
            self.PROVIDER_SETTINGS = self.AGENT_CONFIG["settings"]
            if "provider" in self.PROVIDER_SETTINGS:
                self.AI_PROVIDER = self.PROVIDER_SETTINGS["provider"]
                self.PROVIDER = Provider(self.AI_PROVIDER, **self.PROVIDER_SETTINGS)
            self._load_agent_config_keys(["AI_MODEL", "AI_TEMPERATURE", "MAX_TOKENS"])
            if "AI_MODEL" in self.PROVIDER_SETTINGS:
                self.AI_MODEL = self.PROVIDER_SETTINGS["AI_MODEL"]
                if self.AI_MODEL == "":
                    self.AI_MODEL = "default"
            else:
                self.AI_MODEL = "openassistant"
            if "embedder" in self.PROVIDER_SETTINGS:
                self.EMBEDDER = self.PROVIDER_SETTINGS["embedder"]
            else:
                if self.AI_PROVIDER == "openai":
                    self.EMBEDDER = "openai"
                else:
                    self.EMBEDDER = "default"
            if "MAX_TOKENS" in self.PROVIDER_SETTINGS:
                self.MAX_TOKENS = self.PROVIDER_SETTINGS["MAX_TOKENS"]
            else:
                self.MAX_TOKENS = 4000
            if "LOG_REQUESTS" in self.PROVIDER_SETTINGS:
                self.LOG_REQUESTS = self.PROVIDER_SETTINGS["LOG_REQUESTS"]
            else:
                self.LOG_REQUESTS = True

        # Yaml Memory
        self.memory_file = f"agents/{self.agent_name}.yaml"
        self._create_parent_directories(self.memory_file)
        self.memory = self.load_memory()
        self.agent_instances = {}
        self.agent_config = self.load_agent_config(self.agent_name)
        self.commands = self.load_commands()
        if self.LOG_REQUESTS:
            Path(
                os.path.join(
                    "agents",
                    self.agent_name,
                    "requests",
                )
            ).mkdir(parents=True, exist_ok=True)

    def get_memories(self):
        return Memories(self.agent_name, self.AGENT_CONFIG)

    async def execute(self, command_name, command_args):
        return await Extensions(self.AGENT_CONFIG).execute_command(
            command_name=command_name, command_args=command_args, agent=self
        )

    def instruct(self, prompt, tokens):
        """
        This function takes a prompt and tokens as input, sends the prompt to a provider for a response,
        logs the request if enabled, and returns the response.

        :param prompt: a string representing the user's input or request to the coding assistant
        :param tokens: tokens is a list of strings that represent additional information or context that can
        be used to generate a response to the prompt. These tokens can be used by the PROVIDER to better
        understand the user's intent and provide a more accurate response
        :return: The function `instruct` returns the `answer` variable.
        """
        if not prompt:
            return ""
        answer = self.PROVIDER.instruct(prompt, tokens)
        if self.LOG_REQUESTS:
            log_file = os.path.join(
                "agents", self.agent_name, "requests", f"{time.time()}.txt"
            )
            with open(
                log_file,
                "a" if os.path.exists(log_file) else "w",
                encoding="utf-8",
            ) as f:
                f.write(f"{prompt}\n{answer}")
        return answer

    def _load_agent_config_keys(self, keys):
        """
        This function loads agent configuration keys and sets their values as attributes of the object.

        :param keys: A list of strings representing the keys to be loaded from the agent configuration
        """
        for key in keys:
            if key in self.AGENT_CONFIG:
                setattr(self, key, self.AGENT_CONFIG[key])

    def _create_parent_directories(self, file_path):
        """
        This function creates parent directories for a given file path if they do not already exist.

        :param file_path: The file path is a string that represents the location of a file on the file
        system. It includes the directory path and the file name. For example,
        "/home/user/documents/myfile.txt" is a file path where "/home/user/documents" is the directory path
        and "myfile.txt" is the
        """
        path = Path(file_path)
        path.parent.mkdir(parents=True, exist_ok=True)

    def clean_agent_config_commands(self):
        """
        This function cleans up the agent configuration commands by adding missing commands and removing
        unnecessary ones.
        """
        for command in self.commands:
            friendly_name = command[0]
            if friendly_name not in self.AGENT_CONFIG["commands"]:
                self.AGENT_CONFIG["commands"][friendly_name] = False
        for command in list(self.AGENT_CONFIG["commands"]):
            if command not in [cmd[0] for cmd in self.commands]:
                del self.AGENT_CONFIG["commands"][command]
        with open(f"agents/{self.agent_name}/config.json", "w") as f:
            json.dump(self.AGENT_CONFIG, f)

    def get_commands_string(self):
        """
        This function returns a string containing a list of available commands for completing a task.
        :return: The function `get_commands_string` returns a string that lists the available commands to
        complete a task. If there are no available commands or all available commands are disabled, it
        returns `None`. Otherwise, it returns a string that lists the friendly name, name, and arguments of
        each enabled command.
        """
        if len(self.available_commands) == 0:
            return None

        enabled_commands = filter(
            lambda command: command.get("enabled", True), self.available_commands
        )
        if not enabled_commands:
            return None

        friendly_names = map(
            lambda command: f"{command['friendly_name']} - {command['name']}({command['args']})",
            enabled_commands,
        )
        command_list = "\n".join(friendly_names)
        return f"Commands Available To Complete Task:\n{command_list}\n\n"

    def get_provider(self):
        """
        This function returns the provider specified in the agent configuration file or "openai" if not
        specified.
        :return: The function `get_provider` returns the value of the key "provider" from the agent
        configuration file if it exists, otherwise it returns the string "openai".
        """
        config_file = self.get_agent_config()
        if "provider" in config_file:
            return config_file["provider"]
        else:
            return "openai"

    def create_agent_folder(self, agent_name):
        """
        This function creates a folder for a given agent name in the "agents" directory if it does not
        already exist.

        :param agent_name: The name of the agent for which a folder needs to be created
        :return: the path of the agent folder that was created or already exists.
        """
        agent_folder = f"agents/{agent_name}"
        if not os.path.exists("agents"):
            os.makedirs("agents")
        if not os.path.exists(agent_folder):
            os.makedirs(agent_folder)
        return agent_folder

    def get_command_params(self, func):
        """
        This function retrieves the parameters and default values of a given function.

        :param func: a function object for which we want to retrieve the parameters and their default values
        :return: The function `get_command_params` returns a dictionary containing the parameters of a given
        function `func`. The keys of the dictionary are the parameter names, and the values are either
        `None` if the parameter has no default value, or the default value of the parameter if it has one.
        """
        params = {}
        sig = signature(func)
        for name, param in sig.parameters.items():
            if param.default == Parameter.empty:
                params[name] = None
            else:
                params[name] = param.default
        return params

    def load_commands(self):
        """
        This function loads commands from Python files in a specific directory and returns a list of tuples
        containing information about each command.
        :return: The function `load_commands` returns a list of tuples, where each tuple contains the name
        of a command, the name of the function that implements the command, and a list of parameters that
        the command function takes.
        """
        commands = []
        command_files = glob.glob("extensions/*.py")
        for command_file in command_files:
            module_name = os.path.splitext(os.path.basename(command_file))[0]
            module = importlib.import_module(f"extensions.{module_name}")
            command_class = getattr(module, module_name.lower())()
            if hasattr(command_class, "commands"):
                for command_name, command_function in command_class.commands.items():
                    params = self.get_command_params(command_function)
                    commands.append((command_name, command_function.__name__, params))
        return commands

    def create_agent_config_file(self, agent_name, provider_settings, commands):
        """
        This function creates a configuration file for a given agent with specified provider settings and
        commands.

        :param agent_name: a string representing the name of the agent being created
        :param provider_settings: A dictionary containing the settings for the agent's provider. If it is
        None, an empty string, or an empty dictionary, the default settings will be used
        :param commands: a list of commands that the agent can execute
        :return: the path of the agent config file that was created.
        """
        agent_dir = os.path.join("agents", agent_name)
        agent_config_file = os.path.join(agent_dir, "config.json")
        if (
            provider_settings is None
            or provider_settings == ""
            or provider_settings == {}
        ):
            provider_settings = DEFAULT_SETTINGS
        settings = json.dumps(
            {
                "commands": commands,
                "settings": provider_settings,
            }
        )

        # Check and create agent directory if it doesn't exist
        if not os.path.exists(agent_dir):
            os.makedirs(agent_dir)

        # Write the settings to the agent config file
        with open(agent_config_file, "w") as f:
            f.write(settings)

        return agent_config_file

    def load_agent_config(self, agent_name):
        """
        This function loads the configuration data for a given agent, and if the configuration file is
        missing or invalid, it creates a new one with default settings and all commands disabled.

        :param agent_name: The name of the agent whose configuration is being loaded
        :return: the agent configuration data as a dictionary. If the configuration file does not exist or
        is invalid, it creates a new configuration file with all commands disabled and default settings, and
        returns the new configuration data. If there is an error while creating the new configuration file,
        it returns an empty dictionary.
        """
        try:
            with open(
                os.path.join("agents", agent_name, "config.json")
            ) as agent_config:
                try:
                    agent_config_data = json.load(agent_config)
                    return agent_config_data
                except json.JSONDecodeError:
                    agent_config_data = {}
                    # Populate the agent_config with all commands enabled
                    agent_config_data["commands"] = {
                        command_name: "false"
                        for command_name, _, _ in self.load_commands(agent_name)
                    }
                    agent_config_data["settings"] = DEFAULT_SETTINGS
                    # Save the updated agent_config to the file
                    with open(
                        os.path.join("agents", agent_name, "config.json"), "w"
                    ) as agent_config_file:
                        json.dump(agent_config_data, agent_config_file)
                    return agent_config_data
        except:
            # Add all commands to agent/{agent_name}/config.json in this format {"command_name": "false"}
            agent_config_file = os.path.join("agents", agent_name, "config.json")
            with open(agent_config_file, "w") as f:
                f.write(
                    json.dumps(
                        {
                            "commands": {
                                command_name: "false"
                                for command_name, _, _ in self.load_commands()
                            },
                            "settings": DEFAULT_SETTINGS,
                        }
                    )
                )
        return agent_config_data

    def add_agent(self, agent_name, provider_settings):
        """
        This function adds a new agent with default or provided settings and creates a configuration file
        for the agent.

        :param agent_name: A string representing the name of the agent being added
        :param provider_settings: A dictionary containing settings for the agent's provider. If no settings
        are provided, the function will use default settings
        :return: A dictionary with the key "agent_file" and the value being the name of the agent's YAML
        file.
        """
        if not agent_name:
            return "Agent name cannot be empty."
        provider_settings = (
            DEFAULT_SETTINGS if not provider_settings else provider_settings
        )
        self.create_agent_folder(agent_name)
        commands_list = self.load_commands()
        command_dict = {}
        for command in commands_list:
            friendly_name, command_name, command_args = command
            command_dict[friendly_name] = False
        self.create_agent_config_file(agent_name, provider_settings, command_dict)
        with open(os.path.join("agents", f"{agent_name}.yaml"), "w") as f:
            f.write("")
        return {"agent_file": f"{agent_name}.yaml"}

    def rename_agent(self, agent_name, new_name):
        """
        This function renames an agent by updating the agent's name in a YAML file and renaming the agent's
        folder.

        :param agent_name: The current name of the agent that needs to be renamed
        :param new_name: The new name that the agent will be renamed to
        """
        self.agent_name = new_name
        agent_file = f"agents/{agent_name}.yaml"
        agent_folder = f"agents/{agent_name}/"
        agent_file = os.path.abspath(agent_file)
        agent_folder = os.path.abspath(agent_folder)
        if os.path.exists(agent_file):
            os.rename(agent_file, os.path.join("agents", f"{new_name}.yaml"))
        if os.path.exists(agent_folder):
            os.rename(agent_folder, os.path.join("agents", f"{new_name}"))

    def delete_agent(self, agent_name):
        """
        This function deletes an agent's YAML file and returns an error message if the file is not found.

        :param agent_name: The name of the agent that needs to be deleted
        :return: If the agent file is not found, a dictionary with a "message" key and a 404 status code is
        returned.
        """
        agent_file = f"agents/{agent_name}.yaml"
        agent_folder = f"agents/{agent_name}/"
        agent_file = os.path.abspath(agent_file)
        agent_folder = os.path.abspath(agent_folder)
        try:
            os.remove(agent_file)
        except FileNotFoundError:
            return {"message": f"Agent file {agent_file} not found."}, 404

        if os.path.exists(agent_folder):
            shutil.rmtree(agent_folder)

        return {"message": f"Agent {agent_name} deleted."}, 200

    def get_agent_config(self):
        """
        This function retrieves the configuration file for a specified agent, and if it does not exist,
        creates a new agent with an empty configuration.
        :return: a dictionary object that contains the configuration settings for the agent specified by
        `self.agent_name`. If the configuration file does not exist, the function creates an empty
        configuration file and returns an empty dictionary. If there is an error reading the configuration
        file, the function returns `None`. The function runs in an infinite loop until it is able to
        successfully read the configuration file or create a new one
        """
        while True:
            agent_file = os.path.abspath(f"agents/{self.agent_name}/config.json")
            if os.path.exists(agent_file):
                try:
                    with open(agent_file, "r") as f:
                        file_content = f.read().strip()
                        if file_content:
                            return json.loads(file_content)
                except:
                    None
            self.add_agent(self.agent_name, {})
            return self.get_agent_config()

    def update_agent_config(self, new_config, config_key):
        """
        This function updates the configuration of an agent by modifying a specific key with new values
        while preserving other keys and values.

        :param new_config: A dictionary containing the new configuration values to be updated
        :param config_key: The parameter `config_key` is a string representing the key in the agent's
        configuration file that needs to be updated with the new configuration
        :return: a string message indicating whether the agent configuration was successfully updated or
        not. If the configuration file exists, the function updates the specified key with the new
        configuration and saves the updated configuration back to the file, and returns "Agent {agent_name}
        configuration updated." If the configuration file does not exist, the function returns "Agent
        {agent_name} configuration not found."
        """
        agent_name = self.agent_name
        agent_config_file = os.path.join("agents", agent_name, "config.json")
        if os.path.exists(agent_config_file):
            with open(agent_config_file, "r") as f:
                current_config = json.load(f)

            # Ensure the config_key is present in the current configuration
            if config_key not in current_config:
                current_config[config_key] = {}

            # Update the specified key with new_config while preserving other keys and values
            for key, value in new_config.items():
                current_config[config_key][key] = value

            # Save the updated configuration back to the file
            with open(agent_config_file, "w") as f:
                json.dump(current_config, f)
            return f"Agent {agent_name} configuration updated."
        else:
            return f"Agent {agent_name} configuration not found."

    def get_chat_history(self, agent_name):
        """
        This function retrieves the chat history of a given agent from a YAML file and returns it as a list
        of dictionaries.

        :param agent_name: The name of the agent whose chat history is being retrieved
        :return: a list of dictionaries representing the chat history for a given agent. If the agent's chat
        history file does not exist, an empty list is returned. If there is an error while reading the file,
        an empty list is also returned.
        """
        # If it doesn't exist, create it
        if not os.path.exists(f"agents/{agent_name}.yaml"):
            with open(f"agents/{agent_name}.yaml", "w") as f:
                f.write("")
            return []
        try:
            with open(f"agents/{agent_name}.yaml", "r") as f:
                yaml_history = yaml.safe_load(f)
            chat_history = []
            for interaction in yaml_history["interactions"]:
                role = interaction["role"]
                message = interaction["message"]
                chat_history.append({role: message})
            return chat_history
        except:
            return []

    def wipe_agent_memories(self, agent_name):
        """
        This function deletes the memories folder of a specified agent if it exists.

        :param agent_name: The name of the agent whose memories need to be wiped
        """
        agent_folder = f"agents/{agent_name}/"
        agent_folder = os.path.abspath(agent_folder)
        memories_folder = os.path.join(agent_folder, "memories")
        if os.path.exists(memories_folder):
            shutil.rmtree(memories_folder)

    def load_memory(self):
        """
        This function loads a YAML file containing memory data and returns it, or creates a new file with
        default data if the file does not exist.
        :return: a dictionary object called `memory` which contains a list of interactions. The interactions
        are loaded from a YAML file if it exists, otherwise an empty list is created and saved to the file.
        """
        if os.path.exists(self.memory_file):
            with open(self.memory_file, "r") as file:
                memory = yaml.safe_load(file)
        else:
            with open(self.memory_file, "w") as file:
                yaml.safe_dump({"interactions": []}, file)
            memory = {"interactions": []}
        return memory

    def save_memory(self):
        """
        This Python function saves the memory of an object to a YAML file.
        """
        with open(self.memory_file, "w") as file:
            yaml.safe_dump(self.memory, file)

    def log_interaction(self, role: str, message: str):
        """
        This function logs an interaction by appending the role and message to a memory dictionary and
        saving it.

        :param role: The role of the person or entity that is interacting with the program. It could be a
        user, an administrator, a system, etc
        :type role: str
        :param message: The message parameter is a string that represents the interaction message that is
        being logged. It could be a user's input, a system response, or any other message exchanged during
        the interaction
        :type message: str
        """
        if self.memory is None:
            self.memory = {"interactions": []}
        self.memory["interactions"].append({"role": role, "message": message})
        self.save_memory()



from threading import Thread
from concurrent.futures import Future
import logging
import time
import random


def call_with_future(fn, future, args, kwargs):
    try:
        result = fn(*args, **kwargs)
        future.set_result(result)
    except Exception as exc:
        future.set_exception(exc)


def threaded(fn):
    def wrapper(*args, **kwargs):
        future = Future()
        Thread(target=call_with_future, args=(fn, future, args, kwargs)).start()
        return future

    return wrapper


import requests


class RunpodProvider:
    def __init__(
        self,
        AI_PROVIDER_URI: str = "",
        API_KEY: str = "",
        MAX_TOKENS: int = 2000,
        AI_TEMPERATURE: float = 0.7,
        AI_MODEL: str = "",
        **kwargs,
    ):
        self.requirements = []
        self.AI_PROVIDER_URI = AI_PROVIDER_URI
        self.MAX_TOKENS = MAX_TOKENS
        self.AI_TEMPERATURE = AI_TEMPERATURE
        self.AI_MODEL = AI_MODEL
        self.API_KEY = API_KEY

    @threaded
    def instruct(self, prompt, tokens: int = 0):
        headers = {"Authorization": f"Bearer {self.API_KEY}"}
        max_new_tokens = (
            int(self.MAX_TOKENS) - tokens
            if int(self.MAX_TOKENS) > tokens
            else self.MAX_TOKENS
        )

        logging.info("Instructing Agent with %s", prompt)

        run_response = requests.post(
            f"{self.AI_PROVIDER_URI}/run",
            headers=headers,
            json={
                "input": {
                    "prompt": prompt,
                    "max_new_tokens": max_new_tokens,
                    # "max_context_length": max_tokens,
                    # "max_length": 200,
                    "do_sample": True,
                    "temperature": float(self.AI_TEMPERATURE),
                    "top_p": 0.73,
                    "typical_p": 1,
                    "repetition_penalty": 1.1,
                    "top_k": 0,
                    "min_length": 0,
                    "no_repeat_ngram_size": 0,
                    "num_beams": 1,
                    "penalty_alpha": 0,
                    "length_penalty": 1,
                    "early_stopping": False,
                    "seed": random.randint(1, 1000000000),
                    "add_bos_token": True,
                    "truncation_length": 4096,
                    "ban_eos_token": False,
                    "skip_special_tokens": True,
                    "stopping_strings": [],
                },
            },
        )
        logging.info("Run Response: %s", run_response.json())
        jobId = run_response.json()["id"]
        logging.info("Job ID: %s", jobId)
        while True:
            status_url = f"{self.AI_PROVIDER_URI}/status/{jobId}"
            logging.info("Requesting status url: %s", status_url)
            status_response = requests.get(status_url, headers=headers)
            logging.info("Status Response: %s", status_response.json())
            status = status_response.json()["status"]
            logging.info("Status: %s", status)
            # IN_QUEUE, RUNNING, COMPLETED, FAILED
            if status == "COMPLETED":
                # url = response.json()["results"]["url"]
                # logging.info("Result URL: %s", status)
                # result_response = requests.get(
                #     f"{self.AI_PROVIDER_URI}/status/{jobId}", headers=headers
                # )
                # logging.info("Result: %s", result_response.json())
                output = status_response.json()["output"]
                logging.info("Output: %s", output)
                return output
            elif status == "FAILED":
                logging.info("JOB FAILD - NEEDS HANDLING")
                return None
            else:
                logging.info("Sleeping for 2")
                time.sleep(2)



import streamlit as st
import bcrypt
from auth_libs.Users import load_users, save_user_data
from auth_libs.Cfig import Cfig
from auth_libs.Users import check_auth_status, logout_button
from components.agent_selector import agent_selector
import logging


# Check if the user is logged in
st.session_state["login_page"] = "Profile_Page"
check_auth_status()
st.session_state["login_page"] = False
agent_name, agent = agent_selector()
CFIG = Cfig()


# Toggle public registrations
def toggle_value(val, to_toggle=0):
    setup_CFIG = CFIG.load_config()
    setup_CFIG["config"][to_toggle] = val
    CFIG.save_config(setup_CFIG)

    if val == False:
        st.success("Setting Disabled!")
    else:
        st.success("Setting Enabled!")

    st.session_state["regCheck"] = val


# Change password form
def change_password_form():
    st.write("Change Password")
    current_password = st.text_input("Current Password", type="password")
    new_password = st.text_input("New Password", type="password")
    confirm_password = st.text_input("Confirm New Password", type="password")
    if st.button("Change Password"):
        user_data = load_users()
        for user in user_data["users"]:
            if user["email"] == st.session_state["email"]:
                if bcrypt.checkpw(
                    current_password.encode("utf-8"),
                    user["password_hash"].encode("utf-8"),
                ):
                    if new_password == confirm_password:
                        hashed_password = bcrypt.hashpw(
                            new_password.encode("utf-8"), bcrypt.gensalt()
                        )
                        user["password_hash"] = hashed_password.decode("utf-8")
                        save_user_data(user_data)
                        st.success("Password changed successfully!")
                    else:
                        st.error("New password and confirm password do not match.")
                else:
                    st.error("Incorrect current password.")
                break


# Create user form (admin only)
def create_user_form():
    admin_email = CFIG.get_admin_email()
    if not admin_email:
        st.write("You are not logged in, or login is not enabled.")
    if st.session_state["email"] != admin_email:
        st.error("Only the admin can create new users.")
        return
    st.write("Create User")
    email = st.text_input("Email")
    password = st.text_input("Password", type="password")
    confirm_password = st.text_input("Confirm Password", type="password")
    if st.button("Create User"):
        if not email:
            st.error("Email is required.")
        elif password != confirm_password:
            st.error("Password and confirm password do not match.")
        else:
            user_data = load_users()
            for user in user_data["users"]:
                if user["email"] == email:
                    st.error("User with the same email already exists.")
                    return

            hashed_password = bcrypt.hashpw(password.encode("utf-8"), bcrypt.gensalt())
            new_user = {
                "email": email,
                "password_hash": hashed_password.decode("utf-8"),
                "admin": "False",
            }
            user_data["users"].append(new_user)
            save_user_data(user_data)
            st.success("User created successfully!")

    admin_email = CFIG.get_admin_email()


admin_email = CFIG.get_admin_email()
if (
    st.session_state["email"] == admin_email
    and CFIG.load_config()["auth_setup_config"] != "SUL"
):
    setup_CFIG = CFIG.load_config()
    if not setup_CFIG["config"]:
        allow_reg = False
        allow_uac = False
        allow_ucp = False
        allow_uaa = False
    else:
        config = setup_CFIG["config"]
        allow_reg = config[0]
        allow_uac = config[1]
        allow_ucp = config[2]
        allow_uaa = config[3]

    st.session_state["regCheck"] = allow_reg

    if allow_reg == True:
        ar_opposite = False
    else:
        ar_opposite = True

    if allow_uac == True:
        uac_opposite = False
    else:
        uac_opposite = True

    if allow_ucp == True:
        ucp_opposite = False
    else:
        ucp_opposite = True

    if allow_uaa == True:
        uaa_opposite = False
    else:
        uaa_opposite = True

    col1, col2 = st.columns(2)

    with col1:
        if setup_CFIG["auth_setup_config"] == "Multi-User Public Registration":
            st.checkbox(
                "Enable Public Registration",
                value=allow_reg,
                on_change=toggle_value,
                args=(ar_opposite, 0),
            )
        else:
            st.checkbox(
                "Enable Public Registration",
                value=allow_reg,
                on_change=toggle_value,
                args=(ar_opposite, 0),
                disabled=True,
            )
        st.checkbox(
            "Enable Users to Create Public Agents",
            value=allow_uac,
            on_change=toggle_value,
            args=(uac_opposite, 1),
        )

    with col2:
        st.checkbox(
            "Enable Users To Create Private Agents",
            value=allow_ucp,
            on_change=toggle_value,
            args=(ucp_opposite, 2),
        )
        st.checkbox(
            "Enable Users To Use Admin's Agents",
            value=allow_uaa,
            on_change=toggle_value,
            args=(uaa_opposite, 3),
        )

    change_password_form()
    create_user_form()
else:
    st.write("You are not logged in, or login is not enabled.")



import requests
import inspect
from chromadb.utils import embedding_functions
from chromadb.api.types import Documents, EmbeddingFunction, Embeddings
from semantic_kernel.connectors.ai.hugging_face import HuggingFaceTextEmbedding
from semantic_kernel.connectors.ai.open_ai import (
    AzureTextEmbedding,
    OpenAITextEmbedding,
)
import logging


class GooglePalmEmbeddingFunction(EmbeddingFunction):
    """To use this EmbeddingFunction, you must have the google.generativeai Python package installed and have a PaLM API key."""

    def __init__(self, api_key: str, model_name: str = "models/embedding-gecko-001"):
        if not api_key:
            raise ValueError("Please provide a PaLM API key.")

        if not model_name:
            raise ValueError("Please provide the model name.")

        try:
            import google.generativeai as palm
        except ImportError:
            raise ValueError(
                "The Google Generative AI python package is not installed. Please install it with `pip install google-generativeai`"
            )

        palm.configure(api_key=api_key)
        self._palm = palm
        self._model_name = model_name

    def __call__(self, texts: Documents) -> Embeddings:
        return [
            self._palm.generate_embeddings(model=self._model_name, text=text)[
                "embedding"
            ]
            for text in texts
        ]


class GoogleVertexEmbeddingFunction(EmbeddingFunction):
    # Follow API Quickstart for Google Vertex AI
    # https://cloud.google.com/vertex-ai/docs/generative-ai/start/quickstarts/api-quickstart
    # Information about the text embedding modules in Google Vertex AI
    # https://cloud.google.com/vertex-ai/docs/generative-ai/embeddings/get-text-embeddings
    def __init__(
        self,
        api_key: str,
        model_name: str = "textembedding-gecko-001",
        project_id: str = "cloud-large-language-models",
        region: str = "us-central1",
    ):
        self._api_url = f"https://{region}-aiplatform.googleapis.com/v1/projects/{project_id}/locations/{region}/endpoints/{model_name}:predict"
        self._session = requests.Session()
        self._session.headers.update({"Authorization": f"Bearer {api_key}"})

    def __call__(self, texts: Documents) -> Embeddings:
        response = self._session.post(
            self._api_url, json={"instances": [{"content": texts}]}
        ).json()

        if "predictions" in response:
            return response["predictions"]
        return {}


class LlamacppEmbeddingFunction(EmbeddingFunction):
    def __init__(self, api_host: str):
        self._api_host = api_host
        self._session = requests.Session()

    def __call__(self, texts: Documents) -> Embeddings:
        response = self._session.post(
            self._api_url, json={"content": texts, "threads": 5}
        ).json()
        if "data" in response:
            if "embedding" in response["data"]:
                return response["data"]["embedding"]
        return {}


class Embedding:
    def __init__(self, AGENT_CONFIG=None):
        self.AGENT_CONFIG = AGENT_CONFIG

    async def get_embedder(self):
        try:
            embedder = self.AGENT_CONFIG["settings"]["embedder"]
            embed, chunk_size = await self.__getattribute__(embedder)()
        except:
            embed, chunk_size = await self.default()
            logging.info("Embedder not found, using default embedder")
        return embed, chunk_size

    async def embed_text(self, text):
        embed, chunk_size = await self.get_embedder()
        return await embed(text)

    async def default(self):
        chunk_size = 128
        embed = HuggingFaceTextEmbedding(
            model_name="all-mpnet-base-v2", log=logging
        ).generate_embeddings_async
        return embed, chunk_size

    async def large_local(self):
        chunk_size = 500
        embed = HuggingFaceTextEmbedding(
            model_name="gtr-t5-large", log=logging
        ).generate_embeddings_async
        return embed, chunk_size

    async def azure(self):
        chunk_size = 1000
        embed = AzureTextEmbedding(
            deployment_name=self.AGENT_CONFIG["settings"]["AZURE_DEPLOYMENT_NAME"],
            endpoint=self.AGENT_CONFIG["settings"]["AZURE_OPENAI_ENDPOINT"],
            api_key=self.AGENT_CONFIG["settings"]["AZURE_API_KEY"],
            logger=logging,
        ).generate_embeddings_async
        return embed, chunk_size

    async def openai(self):
        chunk_size = 1000
        embed = OpenAITextEmbedding(
            model_id="text-embedding-ada-002",
            api_key=self.AGENT_CONFIG["settings"]["OPENAI_API_KEY"],
            log=logging,
        ).generate_embeddings_async
        return embed, chunk_size

    async def google_palm(self):
        chunk_size = 1000
        embed = GooglePalmEmbeddingFunction(
            api_key=self.AGENT_CONFIG["settings"]["GOOGLE_API_KEY"],
        )
        return embed, chunk_size

    async def google_vertex(self):
        chunk_size = 1000
        embed = GoogleVertexEmbeddingFunction(
            api_key=self.AGENT_CONFIG["settings"]["GOOGLE_API_KEY"],
            project_id=self.AGENT_CONFIG["settings"]["GOOGLE_PROJECT_ID"],
        )
        return embed, chunk_size

    async def cohere(self):
        chunk_size = 500
        embed = embedding_functions.CohereEmbeddingFunction(
            api_key=self.AGENT_CONFIG["settings"]["COHERE_API_KEY"],
        )
        return embed, chunk_size

    async def llamacpp(self):
        chunk_size = 250
        embed = LlamacppEmbeddingFunction(
            model_name=self.AGENT_CONFIG["settings"]["EMBEDDING_URI"],
        )
        return embed, chunk_size


def get_embedding_providers():
    return [
        func
        for func, _ in inspect.getmembers(Embedding, predicate=inspect.isfunction)
        if not func.startswith("__")
    ]



from typing import List
import spacy
import os
from hashlib import sha256
from Embedding import Embedding
from datetime import datetime
from collections import Counter
import pandas as pd
import docx2txt
import pdfplumber
from playwright.async_api import async_playwright
from semantic_kernel.connectors.memory.chroma import ChromaMemoryStore
from semantic_kernel.memory.memory_record import MemoryRecord
from chromadb.config import Settings
from bs4 import BeautifulSoup
import logging
import asyncio
import sys

if sys.platform == "win32":
    asyncio.set_event_loop_policy(asyncio.WindowsProactorEventLoopPolicy())


class Memories:
    def __init__(self, agent_name: str = "AGiXT", agent_config=None):
        self.agent_name = agent_name
        self.agent_config = agent_config
        self.chroma_client = None
        self.collection = None
        self.nlp = None
        self.chunk_size = 128
        memories_dir = os.path.join(os.getcwd(), "agents", self.agent_name, "memories")
        self.chroma_client = ChromaMemoryStore(
            persist_directory=memories_dir,
            client_settings=Settings(
                chroma_db_impl="duckdb+parquet",
                persist_directory=memories_dir,
                anonymized_telemetry=False,
            ),
        )

    def load_spacy_model(self):
        if not self.nlp:
            try:
                self.nlp = spacy.load("en_core_web_sm")
            except:
                spacy.cli.download("en_core_web_sm")
                self.nlp = spacy.load("en_core_web_sm")
        self.nlp.max_length = 99999999999999999999999

    async def get_embedder(self):
        embedder, chunk_size = await Embedding(
            AGENT_CONFIG=self.agent_config
        ).get_embedder()
        return embedder, chunk_size

    async def get_collection(self):
        try:
            memories_exist = await self.chroma_client.does_collection_exist_async(
                "memories"
            )
            if not memories_exist:
                await self.chroma_client.create_collection_async(
                    collection_name="memories"
                )
                memories = await self.chroma_client.get_collection_async(
                    collection_name="memories"
                )
            else:
                memories = await self.chroma_client.get_collection_async(
                    collection_name="memories"
                )
            return memories
        except Exception as e:
            raise RuntimeError(f"Unable to initialize chroma client: {e}")

    async def store_memory(
        self, content: str, description: str = None, external_source_name: str = None
    ):
        embedder, chunk_size = await self.get_embedder()
        collection = await self.get_collection()
        record = MemoryRecord(
            is_reference=False,
            id=sha256((content + datetime.now().isoformat()).encode()).hexdigest(),
            text=content,
            timestamp=datetime.now().isoformat(),
            description=description,
            external_source_name=external_source_name,  # URL or File path
            embedding=await embedder(content),
        )

        try:
            await self.chroma_client.upsert_async(
                collection_name="memories",
                record=record,
            )
            self.chroma_client._client.persist()
        except Exception as e:
            logging.info(f"Failed to store memory: {e}")

    async def store_result(
        self, task_name: str, result: str, external_source_name: str = None
    ):
        if result:
            if not isinstance(result, str):
                result = str(result)
            chunks = await self.chunk_content(result, task_name)
            for chunk in chunks:
                await self.store_memory(
                    content=chunk,
                    description=task_name,
                    external_source_name=external_source_name,
                )

    async def context_agent(self, query: str, top_results_num: int) -> List[str]:
        embedder, chunk_size = await self.get_embedder()
        collection = await self.get_collection()
        if collection == None:
            return []
        results = await self.chroma_client.get_nearest_matches_async(
            collection_name="memories",
            embedding=await embedder(query),
            limit=top_results_num,
            min_relevance_score=0.1,
        )
        context = []
        for memory, score in results:
            context.append(memory._text)
        trimmed_context = await self.trim_context(context)
        logging.info(f"CONTEXT: {trimmed_context}")
        context_str = "\n".join(trimmed_context)
        response = f"Context: {context_str}\n\n"
        return response

    async def trim_context(self, context: List[str]) -> List[str]:
        embedder, chunk_size = await self.get_embedder()
        if not self.nlp:
            self.load_spacy_model()
        trimmed_context = []
        total_tokens = 0
        for item in context:
            item_tokens = len(self.nlp(item))
            if total_tokens + item_tokens <= chunk_size:
                trimmed_context.append(item)
                total_tokens += item_tokens
            else:
                break
        return trimmed_context

    def get_keywords(self, query: str):
        """Extract keywords from a query using Spacy's part-of-speech tagging."""
        if not self.nlp:
            self.load_spacy_model()
        doc = self.nlp(query)
        keywords = [
            token.text for token in doc if token.pos_ in {"NOUN", "PROPN", "VERB"}
        ]
        return set(keywords)

    def score_chunk(self, chunk: str, keywords: set):
        """Score a chunk based on the number of query keywords it contains."""
        chunk_counter = Counter(chunk.split())
        score = sum(chunk_counter[keyword] for keyword in keywords)
        return score

    async def chunk_content(
        self, content: str, query: str, overlap: int = 2
    ) -> List[str]:
        embedder, chunk_size = await self.get_embedder()
        if not self.nlp:
            self.load_spacy_model()
        doc = self.nlp(content)
        sentences = list(doc.sents)
        content_chunks = []
        chunk = []
        chunk_len = 0
        keywords = self.get_keywords(query)

        for i, sentence in enumerate(sentences):
            sentence_tokens = len(sentence)
            if chunk_len + sentence_tokens > chunk_size and chunk:
                chunk_text = " ".join(token.text for token in chunk)
                content_chunks.append(
                    (self.score_chunk(chunk_text, keywords), chunk_text)
                )
                chunk = list(sentences[i - overlap : i]) if i - overlap >= 0 else []
                chunk_len = sum(len(s) for s in chunk)
            chunk.extend(sentence)
            chunk_len += sentence_tokens

        if chunk:
            chunk_text = " ".join(token.text for token in chunk)
            content_chunks.append((self.score_chunk(chunk_text, keywords), chunk_text))

        # Sort the chunks by their score in descending order before returning them
        content_chunks.sort(key=lambda x: x[0], reverse=True)
        return [chunk_text for score, chunk_text in content_chunks]

    async def mem_read_file(self, file_path: str):
        try:
            # If file extension is pdf, convert to text
            if file_path.endswith(".pdf"):
                with pdfplumber.open(file_path) as pdf:
                    content = "\n".join([page.extract_text() for page in pdf.pages])
            # If file extension is xls, convert to csv
            elif file_path.endswith(".xls") or file_path.endswith(".xlsx"):
                content = pd.read_excel(file_path).to_csv()
            # If file extension is doc, convert to text
            elif file_path.endswith(".doc") or file_path.endswith(".docx"):
                content = docx2txt.process(file_path)
            # TODO: If file is an image, classify it in text.
            # Otherwise just read the file
            else:
                with open(file_path, "r") as f:
                    content = f.read()
            await self.store_result(task_name=file_path, result=content)
            return True
        except:
            return False

    async def read_website(self, url):
        try:
            async with async_playwright() as p:
                browser = await p.chromium.launch()
                context = await browser.new_context()
                page = await context.new_page()
                await page.goto(url)
                content = await page.content()

                # Scrape links and their titles
                links = await page.query_selector_all("a")
                link_list = []
                for link in links:
                    title = await page.evaluate("(link) => link.textContent", link)
                    href = await page.evaluate("(link) => link.href", link)
                    link_list.append((title, href))

                await browser.close()
                soup = BeautifulSoup(content, "html.parser")
                text_content = soup.get_text()
                text_content = " ".join(text_content.split())
                if text_content:
                    await self.store_result(url, text_content)
                return text_content, link_list
        except:
            return None, None



import streamlit as st
from AGiXT import AGiXT
from streamlit import (
    markdown,
    header,
    checkbox,
    container,
    text_input,
    button,
    spinner,
    error,
    warning,
)
import asyncio
from auth_libs.Users import check_auth_status
from components.agent_selector import agent_selector

check_auth_status()
agent_name, agent = agent_selector()


def render_chat_history(chat_container, chat_history):
    chat_container.empty()
    with chat_container:
        for chat in chat_history:
            if "sender" in chat and "message" in chat:
                if chat["sender"] == "User":
                    markdown(
                        f'<div style="text-align: left; margin-bottom: 5px;"><strong>User:</strong> {chat["message"]}</div>',
                        unsafe_allow_html=True,
                    )
                else:
                    markdown(
                        f'<div style="text-align: left; margin-bottom: 5px;"><strong>Agent:</strong> {chat["message"]}</div>',
                        unsafe_allow_html=True,
                    )


header("Chat with Agent")

smart_chat_toggle = checkbox("Enable Smart Chat")

if "chat_history" not in st.session_state:
    st.session_state["chat_history"] = {}

chat_container = container()

if agent_name:
    try:
        st.session_state.chat_history[agent_name] = agent.get_chat_history(agent_name)
    except:
        st.session_state.chat_history[
            agent_name
        ] = []  # initialize as an empty list, not a dictionary

    render_chat_history(chat_container, st.session_state.chat_history[agent_name])

    chat_prompt = text_input("Enter your message", key="chat_prompt")
    send_button = button("Send Message")

    if send_button:
        if agent_name and chat_prompt:
            with spinner("Thinking, please wait..."):
                agent = AGiXT(agent_name=agent_name)
                if smart_chat_toggle:
                    response = asyncio.run(
                        agent.smart_chat(
                            chat_prompt,
                            shots=3,
                        )
                    )
                else:
                    response = asyncio.run(
                        agent.run(
                            chat_prompt,
                            prompt="Chat",
                            context_results=6,
                        )
                    )
            chat_entry = [
                {"sender": "User", "message": chat_prompt},
                {"sender": "Agent", "message": response},
            ]
            st.session_state.chat_history[agent_name].extend(chat_entry)
            render_chat_history(
                chat_container, st.session_state.chat_history[agent_name]
            )
        else:
            error("Agent name and message are required.")
else:
    warning("Please select an agent to start chatting.")



import os
import yaml

CONFIG_FILE = "config.yaml"


class Cfig:
    def __init__(self):
        if not os.path.exists(CONFIG_FILE):
            self.create_config_file()

    def create_config_file(self):
        """
        Creates a default configuration file if it doesn't exist.
        """
        default_config = {"auth_setup": False, "auth_setup_config": None}

        with open(CONFIG_FILE, "w") as f:
            yaml.dump(default_config, f)

    def load_config(self):
        """
        Loads the configuration data from the config file.
        If the file doesn't exist, creates a default file and loads the data.
        Returns the loaded configuration data.
        """
        if not os.path.exists(CONFIG_FILE):
            self.create_config_file()

        with open(CONFIG_FILE, "r") as f:
            config = yaml.safe_load(f)

        return config

    def save_config(self, config):
        """
        Saves the configuration data to the config file.
        """
        with open(CONFIG_FILE, "w") as f:
            yaml.dump(config, f)

    def is_auth_setup_configured(self):
        """
        Checks if the admin configuration setup is already done.
        Returns True if configured, False otherwise.
        """
        config = self.load_config()
        return config["auth_setup"]

    def set_auth_setup_config(self, setup_config):
        """
        Sets the authentication setup configuration.
        """
        config = self.load_config()
        config["auth_setup_config"] = setup_config
        config["auth_setup"] = True
        self.save_config(config)

    def get_admin_email(self):
        config = self.load_config()
        if "admin_email" in config:
            return config["admin_email"]
        else:
            return False



import subprocess
import sys

try:
    from nomic.gpt4all import GPT4All
except ImportError:
    subprocess.check_call([sys.executable, "-m", "pip", "install", "nomic"])
    from nomic.gpt4all import GPT4All


class Gpt4allProvider:
    def __init__(
        self,
        MODEL_NAME: str = "gpt4all-lora-quantized",
        MAX_TOKENS: int = 2000,
        AI_MODEL: str = "default",
        AI_TEMPERATURE: float = 0.7,
        **kwargs,
    ):
        self.model = GPT4All(model=MODEL_NAME)
        self.model.open()
        self.MAX_TOKENS = MAX_TOKENS
        self.AI_MODEL = AI_MODEL
        self.AI_TEMPERATURE = AI_TEMPERATURE
        # TODO: Need to research to add temperature, no obvious flag.

    def instruct(self, prompt):
        try:
            return self.model.prompt(prompt)
        except Exception as e:
            return f"GPT4ALL Error: {e}"



import subprocess
import sys
import os
import logging

try:
    from llama_cpp import Llama
except:
    subprocess.check_call([sys.executable, "-m", "pip", "install", "llama-cpp-python"])
    from llama_cpp import Llama


class LlamacppProvider:
    def __init__(
        self,
        MODEL_PATH: str = "",
        MAX_TOKENS: int = 2000,
        AI_TEMPERATURE: float = 0.7,
        AI_MODEL: str = "default",
        GPU_LAYERS: int = 0,
        BATCH_SIZE: int = 512,
        THREADS: int = 0,
        STOP_SEQUENCE: str = "</s>",
        **kwargs,
    ):
        self.requirements = ["llama-cpp-python"]
        self.AI_TEMPERATURE = AI_TEMPERATURE
        self.MAX_TOKENS = MAX_TOKENS
        self.AI_MODEL = AI_MODEL
        self.GPU_LAYERS = GPU_LAYERS
        self.BATCH_SIZE = BATCH_SIZE
        self.THREADS = THREADS if THREADS != 0 else None
        self.STOP_SEQUENCE = STOP_SEQUENCE
        self.MODEL_PATH = MODEL_PATH
        if self.MODEL_PATH:
            try:
                self.MAX_TOKENS = int(self.MAX_TOKENS)
            except:
                self.MAX_TOKENS = 2000

    def instruct(self, prompt, tokens: int = 0):
        max_tokens = int(self.MAX_TOKENS) - tokens
        if max_tokens < 1:
            max_tokens = int(self.MAX_TOKENS)
        if os.path.isfile(self.MODEL_PATH):
            self.model = Llama(
                model_path=self.MODEL_PATH,
                n_gpu_layers=int(self.GPU_LAYERS),
                n_threads=int(self.THREADS),
                n_ctx=max_tokens,
            )
        else:
            logging.info("Unable to find model path.")
            return None
        response = self.model(
            prompt,
            stop=[self.STOP_SEQUENCE],
            temperature=float(self.AI_TEMPERATURE),
            echo=True,
        )
        data = response["choices"][0]["text"]
        data = data.replace(prompt, "")
        data = data.lstrip("\n")
        return data



import os
import os.path
from typing import List
from Extensions import Extensions
import os
import subprocess
import docker
from docker.errors import ImageNotFound
import logging


class file_system(Extensions):
    def __init__(
        self,
        WORKING_DIRECTORY: str = "./WORKSPACE",
        WORKING_DIRECTORY_RESTRICTED: bool = True,
        **kwargs,
    ):
        self.WORKING_DIRECTORY = WORKING_DIRECTORY
        self.WORKING_DIRECTORY_RESTRICTED = WORKING_DIRECTORY_RESTRICTED
        if not os.path.exists(self.WORKING_DIRECTORY):
            os.makedirs(self.WORKING_DIRECTORY)
        self.commands = {
            "Read File": self.read_file,
            "Write to File": self.write_to_file,
            "Append to File": self.append_to_file,
            "Delete File": self.delete_file,
            "Search Files": self.search_files,
            "Execute Python File": self.execute_python_file,
            "Execute Shell": self.execute_shell,
        }
        self.WORKING_DIRECTORY = WORKING_DIRECTORY

    async def execute_python_file(self, file: str):
        logging.info(f"Executing file '{file}' in workspace '{self.WORKING_DIRECTORY}'")

        if not file.endswith(".py"):
            return "Error: Invalid file type. Only .py files are allowed."

        file_path = os.path.join(self.WORKING_DIRECTORY, file)

        if not os.path.isfile(file_path):
            return f"Error: File '{file}' does not exist."

        if self.we_are_running_in_a_docker_container():
            result = subprocess.run(
                f"python {file_path}", capture_output=True, encoding="utf8", shell=True
            )
            if result.returncode == 0:
                return result.stdout
            else:
                return f"Error: {result.stderr}"

        try:
            client = docker.from_env()

            image_name = "python:3.10"
            try:
                client.images.get(image_name)
                logging.info(f"Image '{image_name}' found locally")
            except ImageNotFound:
                logging.info(
                    f"Image '{image_name}' not found locally, pulling from Docker Hub"
                )
                low_level_client = docker.APIClient()
                for line in low_level_client.pull(image_name, stream=True, decode=True):
                    status = line.get("status")
                    progress = line.get("progress")
                    if status and progress:
                        logging.info(f"{status}: {progress}")
                    elif status:
                        logging.info(status)

            container = client.containers.run(
                image_name,
                f"python {file}",
                volumes={
                    os.path.abspath(self.WORKING_DIRECTORY): {
                        "bind": "/workspace",
                        "mode": "ro",
                    }
                },
                working_dir="/workspace",
                stderr=True,
                stdout=True,
                detach=True,
            )

            container.wait()
            logs = container.logs().decode("utf-8")
            container.remove()

            return logs

        except Exception as e:
            return f"Error: {str(e)}"

    async def execute_shell(self, command_line: str) -> str:
        current_dir = os.getcwd()
        os.chdir(current_dir)
        logging.info(
            f"Executing command '{command_line}' in working directory '{os.getcwd()}'"
        )
        result = subprocess.run(command_line, capture_output=True, shell=True)
        output = f"STDOUT:\n{result.stdout}\nSTDERR:\n{result.stderr}"

        os.chdir(current_dir)

        return output

    @staticmethod
    def we_are_running_in_a_docker_container() -> bool:
        return os.path.exists("/.dockerenv")

    def safe_join(self, base: str, paths) -> str:
        if "/path/to/" in paths:
            paths = paths.replace("/path/to/", "")
        if str(self.WORKING_DIRECTORY_RESTRICTED).lower() == "true":
            new_path = os.path.normpath(os.path.join(base, *paths.split("/")))
            if not os.path.exists(new_path):
                if "." not in new_path:
                    os.makedirs(new_path)
        else:
            new_path = os.path.normpath(os.path.join("/", *paths))
            if not os.path.exists(new_path):
                os.makedirs(new_path)
        return new_path

    async def read_file(self, filename: str) -> str:
        try:
            filepath = self.safe_join(base=self.WORKING_DIRECTORY, paths=filename)
            with open(filepath, "r", encoding="utf-8") as f:
                content = f.read()
            return content
        except Exception as e:
            return f"Error: {str(e)}"

    async def write_to_file(self, filename: str, text: str) -> str:
        try:
            filepath = self.safe_join(base=self.WORKING_DIRECTORY, paths=filename)
            directory = os.path.dirname(filepath)
            if not os.path.exists(directory):
                os.makedirs(directory)
            with open(filepath, "w", encoding="utf-8") as f:
                f.write(text)
            return "File written to successfully."
        except Exception as e:
            return f"Error: {str(e)}"

    async def append_to_file(self, filename: str, text: str) -> str:
        try:
            filepath = self.safe_join(base=self.WORKING_DIRECTORY, paths=filename)
            with open(filepath, "a") as f:
                f.write(text)
            return "Text appended successfully."
        except Exception as e:
            return f"Error: {str(e)}"

    async def delete_file(self, filename: str) -> str:
        try:
            filepath = self.safe_join(base=self.WORKING_DIRECTORY, paths=filename)
            os.remove(filepath)
            return "File deleted successfully."
        except Exception as e:
            return f"Error: {str(e)}"

    async def search_files(self, directory: str) -> List[str]:
        found_files = []

        if directory in {"", "/"}:
            search_directory = self.WORKING_DIRECTORY
        else:
            search_directory = self.safe_join(
                base=self.WORKING_DIRECTORY, paths=directory
            )

        for root, _, files in os.walk(search_directory):
            for file in files:
                if file.startswith("."):
                    continue
                relative_path = os.path.relpath(
                    os.path.join(root, file), self.WORKING_DIRECTORY
                )
                found_files.append(relative_path)

        return found_files



import json
import random
import requests
from typing import List
from Extensions import Extensions


class searxng(Extensions):
    def __init__(self, SEARXNG_INSTANCE_URL: str = "", **kwargs):
        self.SEARXNG_INSTANCE_URL = SEARXNG_INSTANCE_URL
        self.SEARXNG_ENDPOINT = self.get_server()
        self.commands = {"Use The Search Engine": self.search}

    def get_server(self):
        if self.SEARXNG_INSTANCE_URL == "":
            try:  # SearXNG - List of these at https://searx.space/
                response = requests.get("https://searx.space/data/instances.json")
                data = json.loads(response.text)
                servers = list(data["instances"].keys())
                random_index = random.randint(0, len(servers) - 1)
                self.SEARXNG_INSTANCE_URL = servers[random_index]
            except:  # Select default remote server that typically works if unable to get list.
                self.SEARXNG_INSTANCE_URL = "https://search.us.projectsegfau.lt"
        server = self.SEARXNG_INSTANCE_URL.rstrip("/")
        endpoint = f"{server}/search"
        return endpoint

    async def search(self, query: str) -> List[str]:
        try:
            response = requests.get(
                self.SEARXNG_ENDPOINT,
                params={
                    "q": query,
                    "language": "en",
                    "safesearch": 1,
                    "format": "json",
                },
            )
            results = response.json()
            summaries = [
                result["title"] + " - " + result["url"] for result in results["results"]
            ]
            return summaries
        except:
            # The SearXNG server is down or refusing connection, so we will use the default one.
            self.SEARXNG_ENDPOINT = "https://search.us.projectsegfau.lt/search"
            return self.search(query)



import os
import json
import bcrypt
import streamlit as st
from auth_libs.Cfig import Cfig
import auth_libs.Redirect as redir
import logging
import time

CFIG = Cfig()

USER_FILE = "user_data.json"


def load_users():
    """
    Loads the user data from the user file.
    If the user file doesn't exist, triggers the initial user setup.
    Returns the loaded user data.
    """
    if not os.path.exists(USER_FILE):
        # Initial user setup
        st.write("Admin Account Setup")
        admin_email = st.text_input("Enter the admin email")
        admin_password = st.text_input("Enter the admin password", type="password")
        confirm_password = st.text_input("Confirm the admin password", type="password")
        if st.button("Create Admin Account"):
            if admin_password != confirm_password:
                st.error("Password and confirm password do not match.")
            else:
                admin_password_hash = str(
                    bcrypt.hashpw(
                        admin_password.encode("utf-8"), bcrypt.gensalt()
                    ).decode()
                )
                admin_data = {
                    "email": admin_email,
                    "password_hash": admin_password_hash,
                    "admin": True,
                }
                user_data = {"users": [admin_data]}
                save_user_data(user_data)
                config = CFIG.load_config()
                config["admin_email"] = admin_email
                config["config"] = ["", "", "", ""]
                # allow_reg = False
                config["config"][0] = False
                # allow_uac = False
                config["config"][1] = False
                # allow_ucp = False
                config["config"][2] = False
                # allow_uaa = False
                config["config"][3] = False
                CFIG.save_config(config)
                st.success("Admin account created successfully!")
                # Clear session state and redirect to the login page
                st.session_state.clear()
                st.experimental_rerun()
        st.stop()
    else:
        with open(USER_FILE, "r") as file:
            user_data = json.load(file)
    try:
        return user_data
    except:
        return None


def save_user_data(user_data):
    """
    Saves the user data to the user file.
    """
    with open(USER_FILE, "w") as file:
        json.dump(user_data, file, indent=4)


def configure_auth_settings():
    """
    Prompts the admin to configure the authentication settings.
    """
    st.write("Auth/Login Settings Configuration")
    setup_cfg = st.radio(
        "Auth/Login Settings",
        (
            "No Login",
            "Single-User Login",
            "Multi-User Private Registration",
            "Multi-User Public Registration",
        ),
        0,
    )
    if st.button("Build Config"):
        CFIG.set_auth_setup_config(setup_cfg)
        st.success("Auth/Login settings configured!")
        st.experimental_rerun()
    st.stop()


def check_admin_configured():
    """
    Checks if the admin configuration is already done.
    Returns True if configured, False otherwise.
    """
    return CFIG.is_auth_setup_configured()


def logout_button():
    """
    Renders the logout button.
    """
    if st.button("Logout"):
        # Clear session state and redirect to the login page
        st.session_state.clear()
        st.experimental_rerun()  # Redirect to the login page


def check_auth_status():
    # Check if the user is logged in
    if (
        not st.session_state.get("logged_in")
        and os.path.exists("config.yaml")
        and (CFIG.load_config()["auth_setup"] == True)
    ):
        # Redirect to the login page if not
        redir.nav_page("Login")
        logging.info("Not logged in")
    else:
        if CFIG.load_config()["auth_setup"] == True:
            if (
                st.session_state.get("logged_in")
                and st.session_state.get("login_page") == True
            ):
                logging.info("Logged In!")
                redir.nav_page("Profile")
            logout_button()
        elif st.session_state.get("login_page") == "Profile_Page":
            redir.nav_page("Login")



import requests

# Llamacpp API Server
# ./server --model "/path/to/ggml-model.bin" --ctx_size 2048 --ngl 32 -port 7171
# Embedding server
# ./server --model "/path/to/ggml-model.bin" --ctx_size 2048 --ngl 32 -port 7172 --embedding

# ctx_size is max tokens
# ngl is is GPU Layers.  Supposedly this works well for an RTX 3080.

# If using the settings above, use the following in your agent settings:
# AI_PROVIDER_URI = "http://localhost:7171"
# EMBEDDING_URI = "http://localhost:7172"


class LlamacppapiProvider:
    def __init__(
        self,
        AI_PROVIDER_URI: str = "http://localhost:7171",
        EMBEDDING_URI: str = "http://localhost:7172",
        MAX_TOKENS: int = 2000,
        AI_TEMPERATURE: float = 0.7,
        AI_MODEL: str = "default",
        BATCH_SIZE: int = 512,
        THREADS: int = 0,
        STOP_SEQUENCE: str = "\n",
        EXCLUDE_STRING: str = "",
        **kwargs,
    ):
        self.AI_PROVIDER_URI = AI_PROVIDER_URI
        self.AI_TEMPERATURE = AI_TEMPERATURE
        self.MAX_TOKENS = MAX_TOKENS
        self.AI_MODEL = AI_MODEL
        self.BATCH_SIZE = BATCH_SIZE
        self.THREADS = THREADS if THREADS != 0 else None
        self.STOP_SEQUENCE = STOP_SEQUENCE
        self.EXCLUDE_STRING = EXCLUDE_STRING
        self.EMBEDDING_URI = EMBEDDING_URI
        self.MAX_TOKENS = int(self.MAX_TOKENS)

    def instruct(self, prompt, tokens: int = 0):
        new_tokens = int(self.MAX_TOKENS) - tokens
        params = {
            "prompt": prompt,
            "batch_size": int(self.BATCH_SIZE),
            "temperature": float(self.AI_TEMPERATURE),
            "stop": self.STOP_SEQUENCE,
            "exclude": self.EXCLUDE_STRING,
            "n_predict": new_tokens,
            "threads": int(self.THREADS),
            "interactive": False,
        }
        response = requests.post(f"{self.AI_PROVIDER_URI}/completion", json=params)
        data = response.json()
        return data["content"]

    def embeddding(self, prompt):
        data = requests.post(
            f"{self.EMBEDDING_URI}/embeddding",
            json={"content": prompt, "threads": int(self.THREADS)},
        )
        return data["embedding"]



from Extensions import Extensions
import os
import requests
from playsound import playsound


class elevenlabs(Extensions):
    def __init__(
        self,
        ELEVENLABS_API_KEY: str = "",
        ELEVENLABS_VOICE: str = "Josh",
        **kwargs,
    ):
        self.ELEVENLABS_API_KEY = ELEVENLABS_API_KEY
        self.ELEVENLABS_VOICE = ELEVENLABS_VOICE
        self.commands = {"Speak with TTS Using Elevenlabs": self.speak_with_elevenlabs}

    async def speak_with_elevenlabs(self, text: str, voice_index: int = 0) -> bool:
        voices = ["ErXwobaYiN019PkySvjV", "EXAVITQu4vr4xnSDxMaL"]
        headers = {
            "Content-Type": "application/json",
            "xi-api-key": self.ELEVENLABS_VOICE,
        }

        tts_url = f"https://api.elevenlabs.io/v1/text-to-speech/{voices[voice_index]}"
        response = requests.post(tts_url, headers=headers, json={"text": text})

        if response.status_code == 200:
            with open("speech.mpeg", "wb") as f:
                f.write(response.content)
            playsound("speech.mpeg", True)
            os.remove("speech.mpeg")
            return True
        else:
            return False



import gpt4free
import time
import logging
import importlib


class Gpt4freeProvider:
    def __init__(
        self,
        AI_MODEL: str = "gpt-3.5-turbo",
        AI_TEMPERATURE: float = 0.7,
        MAX_TOKENS: int = 4000,
        **kwargs,
    ):
        self.requirements = ["gpt4free"]
        self.AI_MODEL = AI_MODEL
        self.AI_TEMPERATURE = AI_TEMPERATURE
        self.MAX_TOKENS = MAX_TOKENS
        self.FAILED_PROVIDERS = []
        self.providers = ["DeepAI", "You", "UseLess", "ForeFront", "Theb"]
        self.account_tokens = {}

    def instruct(self, prompt, tokens: int = 0):
        final_response = None
        while final_response is None:
            for provider in self.providers:
                try:
                    if provider not in self.FAILED_PROVIDERS:
                        logging.info(f"[GPT4Free] Using: {provider}")
                        if provider not in self.account_tokens:
                            try:
                                if provider == "Poe":
                                    module_name = "quora"
                                elif provider == "UseLess":
                                    module_name = "usesless"
                                else:
                                    module_name = provider.lower()
                                module = importlib.import_module(
                                    "gpt4free.%s" % module_name
                                )
                                if module and hasattr(module, "Account"):
                                    logging.info(f"Create account for: {provider}")
                                    self.account_tokens[
                                        provider
                                    ] = module.Account.create()
                            except ModuleNotFoundError:
                                self.account_tokens[provider] = None
                        args = {}
                        if provider in self.account_tokens:
                            if provider == "ForeFront":
                                args["account_data"] = self.account_tokens[provider]
                            elif provider == "UseLess":
                                args["token"] = self.account_tokens[provider]
                            elif provider == "Poe":
                                args["token"] = self.account_tokens[provider]
                                args["model"] = "GPT-4"

                        response = gpt4free.Completion.create(
                            getattr(gpt4free.Provider, provider), prompt=prompt, **args
                        )
                        if response:
                            if provider == "UseLess":
                                if "text" in response:
                                    response = response["text"]
                                if (
                                    "status" in response
                                    and response["status"] == "Fail"
                                ):
                                    self.FAILED_PROVIDERS.append(provider)
                                    logging.info(
                                        f"Failed to use {provider}: {response}"
                                    )
                                    response = None
                            if (
                                response
                                == "Unable to fetch the response, Please try again."
                            ):
                                self.FAILED_PROVIDERS.append(provider)
                                logging.info(f"Failed to use {provider}: {response}")
                                response = None
                            if final_response == None:
                                final_response = response
                    if final_response:
                        if len(final_response) > 1:
                            return final_response
                except Exception as e:
                    logging.info(f"Failed to use {provider}: {e}")
                    self.FAILED_PROVIDERS.append(provider)
                    final_response = None

            if len(self.FAILED_PROVIDERS) == len(self.providers):
                self.FAILED_PROVIDERS = []
                logging.info(
                    "All providers failed, sleeping for 10 seconds before trying again..."
                )
                time.sleep(10)



from Extensions import Extensions
import os
from playsound import playsound
import gtts as ts


class gtts(Extensions):
    def __init__(
        self,
        USE_GTTS: bool = False,
        **kwargs,
    ):
        self.USE_GTTS = USE_GTTS
        if USE_GTTS:
            self.commands = {"Speak with GTTS": self.speak_with_gtts}

    async def speak_with_gtts(self, text: str) -> bool:
        tts = ts.gTTS(text)
        tts.save("speech.mp3")
        playsound("speech.mp3", True)
        os.remove("speech.mp3")
        return True



from revChatGPT.V1 import Chatbot


# python -m pip install --upgrade revChatGPT
# Get access token from https://chat.openai.com/api/auth/session
class ChatgptProvider:
    def __init__(
        self,
        CHATGPT_TOKEN: str = "",
        AI_MODEL: str = "gpt-3.5-turbo",
        **kwargs,
    ):
        self.requirements = ["revChatGPT"]
        try:
            self.bot = Chatbot(
                config={"access_token": CHATGPT_TOKEN, "model": AI_MODEL}
            )
        except:
            raise Exception(
                "Invalid Chatgpt Token. Get access token from https://chat.openai.com/api/auth/session"
            )
        self.AI_MODEL = AI_MODEL

    def instruct(self, prompt, tokens: int = 0):
        response = ""
        try:
            for data in self.bot.ask(prompt=prompt, model=self.AI_MODEL):
                response = data["message"]
            return response
        except Exception as e:
            return f"Chatgpt Error: {e}"



import requests


class KoboldProvider:
    def __init__(
        self,
        AI_PROVIDER_URI: str = "",
        MAX_TOKENS: int = 2000,
        AI_TEMPERATURE: float = 0.7,
        AI_MODEL: str = "default",
        **kwargs,
    ):
        self.requirements = []
        self.AI_PROVIDER_URI = AI_PROVIDER_URI
        self.MAX_TOKENS = MAX_TOKENS
        self.AI_TEMPERATURE = AI_TEMPERATURE
        self.AI_MODEL = AI_MODEL

    def instruct(self, prompt, tokens: int = 0):
        try:
            max_tokens = int(self.MAX_TOKENS - tokens)
        except:
            max_tokens = 2000
        response = requests.post(
            f"{self.AI_PROVIDER_URI}/api/v1/generate",
            json={
                "prompt": prompt,
                "max_context_length": max_tokens,
                "max_length": 200,
                "temperature": float(self.AI_TEMPERATURE),
            },
        )
        try:
            return response.json()["results"][0]["text"].replace("\n", "\n")
        except:
            return response.json()["detail"][0]["msg"].replace("\n", "\n")



import git
from github import Github
from Extensions import Extensions


class github(Extensions):
    def __init__(
        self,
        GITHUB_USERNAME: str = "",
        GITHUB_API_KEY: str = "",
        WORKING_DIRECTORY: str = "./WORKSPACE",
        **kwargs,
    ):
        self.GITHUB_USERNAME = GITHUB_USERNAME
        self.GITHUB_API_KEY = GITHUB_API_KEY
        self.WORKING_DIRECTORY = WORKING_DIRECTORY
        self.commands = {"Clone Github Repository": self.clone_repo}
        if self.GITHUB_USERNAME and self.GITHUB_API_KEY:
            self.commands["Create Github Repository"] = self.create_repo

    async def clone_repo(self, repo_url: str, clone_path: str) -> str:
        split_url = repo_url.split("//")
        if self.GITHUB_USERNAME is not None and self.GITHUB_API_KEY is not None:
            auth_repo_url = f"//{self.GITHUB_USERNAME}:{self.GITHUB_API_KEY}@".join(
                split_url
            )
        else:
            auth_repo_url = "//".join(split_url)
        try:
            git.Repo.clone_from(auth_repo_url, self.WORKING_DIRECTORY + clone_path)
            return f"""Cloned {repo_url} to {self.WORKING_DIRECTORY + clone_path}"""
        except Exception as e:
            return f"Error: {str(e)}"

    async def create_repo(self, repo_name: str, readme: str) -> str:
        g = Github(self.GITHUB_API_KEY)
        user = g.get_user(self.GITHUB_USERNAME)
        repo = user.create_repo(repo_name, private=True)
        repo_url = repo.clone_url
        repo_dir = f"./{repo_name}"
        repo = git.Repo.init(repo_dir)
        with open(f"{repo_dir}/README.md", "w") as f:
            f.write(readme)
        repo.git.add(A=True)
        repo.git.commit(m="Added README")
        repo.create_remote("origin", repo_url)
        repo.git.push("origin", "HEAD:main")
        return repo_url



import streamlit as st
import auth_libs.Redirect as redir
from Config import Config
from Chain import Chain
from Extensions import Extensions
from Agent import Agent
from Prompts import Prompts
import logging
import asyncio
from auth_libs.Users import check_auth_status
from components.agent_selector import agent_selector

check_auth_status()

CFG = Config()

st.header("Manage Chains")
st.markdown("### Predefined Injection Variables")
st.markdown(
    """
    Any of these variables can be used in command arguments or prompt arguments to inject data into the prompt. These can also be used inside of any Custom Prompt.
- `{agent_name}` will cause the agent name to be injected.
- `{context}` will cause the current context from memory to be injected.
- `{date}` will cause the current date and timestamp to be injected.
- `{COMMANDS}` will cause the available commands list to be injected and for automatic commands execution from the agent based on its suggestions.
- `{command_list}` will cause the available commands list to be injected, but will not execute any commands the AI chooses. Useful on validation steps.
- `{STEPx}` will cause the step `x` response from a chain to be injected. For example, `{STEP1}` will inject the first step's response in a chain.
"""
)

chain_action = st.selectbox("Action", ["Create Chain", "Delete Chain", "Run Chain"])
if chain_action == "Create Chain":
    chain_name = st.text_input("Chain Name")
else:
    chain_name = st.selectbox("Chains", Chain().get_chains())

if st.button("Perform Action"):
    if chain_name:
        if chain_action == "Create Chain":
            Chain().add_chain(chain_name=chain_name)
            st.success(f"Chain '{chain_name}' created.")
            st.experimental_rerun()
        elif chain_action == "Delete Chain":
            Chain().delete_chain(chain_name=chain_name)
            st.success(f"Chain '{chain_name}' deleted.")
            st.experimental_rerun()
        elif chain_action == "Run Chain":
            asyncio.run(Chain().run_chain(chain_name=chain_name))
            st.success(f"Chain '{chain_name}' executed.")
    else:
        st.error("Chain name is required.")

st.header("Manage Chain Steps & View Responses")

chain_names = Chain().get_chains()
selected_chain_name = st.selectbox("Select Chain", [""] + chain_names)

if selected_chain_name:
    try:
        chain = Chain().get_steps(chain_name=selected_chain_name)
    except:
        st.write(selected_chain_name + " Responses: ")
        try:
            if "_responses" in selected_chain_name:
                chain_commands_executed = Chain().get_step_response(
                    chain_name=selected_chain_name.replace("_responses", "")
                )
            else:
                chain_commands_executed = False
            chain_response = Chain().get_step_response(
                chain_name=selected_chain_name,
            )
            if chain_response and chain_commands_executed:
                for exec in chain_commands_executed["steps"]:
                    logging.info("----------exec: " + str(exec["step"]))
                    logging.info("----------Chain_Response " + str(chain_response["1"]))
                    st.write(exec)
                    st.write(chain_response[str(exec["step"])])
            elif chain_response:
                st.write(chain_response)
            else:
                raise ValueError("End of responses!", "None Found!")
        except ValueError as err:
            st.write(err.args)
            loop = False
        st.stop()

    st.subheader(f"Selected Chain: {selected_chain_name}")

    def modify_step(step):
        step_number = step["step"]
        agent_name = step["agent_name"]
        prompt_type = step["prompt_type"]
        prompt = step.get("prompt", "")
        agent_config = Agent(agent_name).agent_config
        modify_step_number = st.number_input(
            "Step Number",
            min_value=1,
            step=1,
            value=step_number,
            key=f"step_num_{step_number}",
        )
        modify_agent_name = st.selectbox(
            "Select Agent",
            options=[""] + [agent["name"] for agent in CFG.get_agents()],
            index=([agent["name"] for agent in CFG.get_agents()].index(agent_name) + 1)
            if agent_name in [agent["name"] for agent in CFG.get_agents()]
            else 0,
            key=f"agent_name_{step_number}",
        )
        modify_prompt_type = st.selectbox(
            "Select Prompt Type",
            options=["", "Command", "Prompt"],
            index=["", "Command", "Prompt"].index(prompt_type),
            key=f"prompt_type_{step_number}",
        )

        if modify_prompt_type == "Command":
            available_commands = [
                cmd["friendly_name"]
                for cmd in Extensions(agent_config).get_enabled_commands()
            ]
            command_name = st.selectbox(
                "Select Command",
                [""] + available_commands,
                key=f"command_name_{step_number}",
                index=available_commands.index(prompt.get("command_name", "")) + 1
                if "command_name" in prompt
                else 0,
            )

            if command_name:
                command_args = Extensions(agent_config).get_command_args(command_name)
                formatted_command_args = ", ".join(
                    [
                        f"{arg}: {st.text_input(arg, value=prompt.get(arg, ''), key=f'{arg}_{step_number}')} "
                        for arg in command_args
                        if arg != "context"
                        and arg != "command_list"
                        and arg != "COMMANDS"
                    ]
                )
                modify_prompt = f"{command_name}({formatted_command_args})"
        elif modify_prompt_type == "Prompt":
            available_prompts = Prompts().get_prompts()
            modify_prompt_name = st.selectbox(
                "Select Custom Prompt",
                [""] + available_prompts,
                key=f"prompt_name_{step_number}",
                index=available_prompts.index(prompt.get("prompt_name", "")) + 1
                if "prompt_name" in prompt
                else 0,
            )

            if modify_prompt_name:
                prompt_args = Prompts().get_prompt_args(modify_prompt_name)
                formatted_prompt_args = ", ".join(
                    [
                        f"{arg}: {st.text_input(arg, value=prompt.get(arg, ''), key=f'{arg}_{step_number}')} "
                        for arg in prompt_args
                        if arg != "context"
                        and arg != "command_list"
                        and arg != "COMMANDS"
                    ]
                )
                modify_prompt = f"{modify_prompt_name}({formatted_prompt_args})"
        else:
            modify_prompt = ""

        if st.button("Modify Step", key=f"modify_{step_number}"):
            Chain().update_step(
                selected_chain_name,
                step_number,
                modify_agent_name,
                modify_prompt_type,
                modify_prompt,
            )
            st.success(f"Step {step_number} updated in chain '{selected_chain_name}'.")
            st.experimental_rerun()
            return {
                "step": modify_step_number,
                "agent_name": modify_agent_name,
                "prompt_type": modify_prompt_type,
                "prompt": modify_prompt,
            }
        return step

    st.write("Existing Steps:")
    if chain:
        for step in chain:
            if step is not None:
                try:
                    step = modify_step(step)
                except TypeError:
                    st.error(
                        "Error loading chain step. Please check the chain configuration."
                    )

    step_number = max([s["step"] for s in chain]) + 1 if chain else 1
    agent_name = st.selectbox(
        "Select Agent",
        options=[""] + [agent["name"] for agent in CFG.get_agents()],
        index=0,
        key="add_step_agent_name",
    )
    prompt_type = st.selectbox(
        "Select Prompt Type",
        [""] + ["Command", "Prompt"],
        key="add_step_prompt_type",
    )

    if prompt_type == "Command":
        agent_config = Agent(agent_name).agent_config
        available_commands = [
            cmd["friendly_name"]
            for cmd in Extensions(agent_config).get_enabled_commands()
        ]
        command_name = st.selectbox(
            "Select Command",
            [""] + available_commands,
            key="add_step_command_name",
        )

        if command_name:
            command_args = Extensions(agent_config).get_command_args(command_name)
            formatted_command_args = ", ".join(
                [
                    f"{arg}: {st.text_input(arg, key=f'add_step_{arg}')} "
                    for arg in command_args
                    if arg != "context" and arg != "command_list" and arg != "COMMANDS"
                ]
            )
            prompt = f"{command_name}({formatted_command_args})"
    elif prompt_type == "Prompt":
        available_prompts = Prompts().get_prompts()
        prompt_name = st.selectbox(
            "Select Custom Prompt",
            [""] + available_prompts,
            key="add_step_prompt_name",
        )

        if prompt_name:
            prompt_args = Prompts().get_prompt_args(prompt_name)
            formatted_prompt_args = ", ".join(
                [
                    f"{arg}: {st.text_input(arg, key=f'add_step_{arg}')} "
                    for arg in prompt_args
                    if arg != "context" and arg != "command_list" and arg != "COMMANDS"
                ]
            )
            prompt = f"{prompt_name}({formatted_prompt_args})"
    else:
        prompt = ""

    step_action = st.selectbox(
        "Action",
        ["Add Step", "Update Step", "Delete Step"],
        key="add_step_action",
    )

    if st.button("Perform Step Action", key="add_step_button"):
        if (
            selected_chain_name
            and step_number
            and agent_name
            and prompt_type
            and prompt
        ):
            if step_action == "Add Step":
                if prompt_type == "Command":
                    prompt_data = {"command_name": command_name}
                    for arg in command_args:
                        if (
                            arg != "context"
                            and arg != "command_list"
                            and arg != "COMMANDS"
                        ):
                            prompt_data[arg] = st.session_state[f"add_step_{arg}"]
                elif prompt_type == "Prompt":
                    prompt_data = {"prompt_name": prompt_name}
                    for arg in prompt_args:
                        if (
                            arg != "context"
                            and arg != "command_list"
                            and arg != "COMMANDS"
                        ):
                            prompt_data[arg] = st.session_state[f"add_step_{arg}"]

                Chain().add_chain_step(
                    chain_name=selected_chain_name,
                    step_number=step_number,
                    agent_name=agent_name,
                    prompt_type=prompt_type,
                    prompt=prompt_data,
                )
                st.success(
                    f"Step {step_number} added to chain '{selected_chain_name}'."
                )
                st.experimental_rerun()
            elif step_action == "Update Step":
                if prompt_type == "Command":
                    prompt_data = {"command_name": command_name}
                    for arg in command_args:
                        if (
                            arg != "context"
                            and arg != "command_list"
                            and arg != "COMMANDS"
                        ):
                            prompt_data[arg] = st.session_state[f"add_step_{arg}"]
                elif prompt_type == "Prompt":
                    prompt_data = {"prompt_name": prompt_name}
                    for arg in prompt_args:
                        if (
                            arg != "context"
                            and arg != "command_list"
                            and arg != "COMMANDS"
                        ):
                            prompt_data[arg] = st.session_state[f"add_step_{arg}"]

                Chain().update_step(
                    selected_chain_name,
                    step_number,
                    agent_name,
                    prompt_type,
                    prompt_data,
                )
                st.success(
                    f"Step {step_number} updated in chain '{selected_chain_name}'."
                )
                st.experimental_rerun()
            elif step_action == "Delete Step":
                Chain().delete_step(selected_chain_name, step_number)
                st.success(
                    f"Step {step_number} deleted from chain '{selected_chain_name}'."
                )
                st.experimental_rerun()
        else:
            st.error("All fields are required.")
else:
    st.warning("Please select a chain to manage steps.")



import streamlit as st
import os
import asyncio
from auth_libs.Users import check_auth_status
from components.agent_selector import agent_selector

check_auth_status()

agent_name, agent = agent_selector()

st.title("Manage Learning")

# Initialize session state for stop events and agent status if not exist
if "agent_status" not in st.session_state:
    st.session_state.agent_status = {}

if agent_name:
    st.markdown("## Learn from a file")
    memories = agent.get_memories()
    learn_file_upload = st.file_uploader(
        "Upload a file for the agent to learn from.", accept_multiple_files=True
    )
    if learn_file_upload is not None:
        for learn_file_upload in learn_file_upload.copy():
            learn_file_path = os.path.join(
                "data", "uploaded_files", learn_file_upload.name
            )
            if not os.path.exists(os.path.dirname(learn_file_path)):
                os.makedirs(os.path.dirname(learn_file_path))
            with open(learn_file_path, "wb") as f:
                f.write(learn_file_upload.getbuffer())
            asyncio.run(memories.mem_read_file(learn_file_path))
            st.success(
                "Agent '"
                + agent_name
                + "' has learned from file: "
                + learn_file_upload.name
            )

    st.markdown("## Learn from a URL")
    learn_url = st.text_input("Enter a URL for the agent to learn from..")
    if st.button("Learn from URL"):
        if learn_url:
            _, _ = asyncio.run(memories.read_website(learn_url))
            st.success(f"Agent '{agent_name}' has learned from the URL.")
    st.markdown("## Wipe Agent Memory")
    st.markdown(
        "The agent can simply learn too much undesired information at times. If you're having an issue with the context being injected from memory with your agent, try wiping the memory."
    )
    if st.button("Wipe agent memory"):
        agent.wipe_agent_memories(agent_name)
        st.success(f"Memory for agent '{agent_name}' has been cleared.")



from streamlit.components.v1 import html


def nav_page(page_name, timeout_secs=3):
    nav_script = """
        <script type="text/javascript">
            function attempt_nav_page(page_name, start_time, timeout_secs) {
                var links = window.parent.document.getElementsByTagName("a");
                for (var i = 0; i < links.length; i++) {
                    if (links[i].href.toLowerCase().endsWith("/" + page_name.toLowerCase())) {
                        links[i].click();
                        return;
                    }
                }
                var elapsed = new Date() - start_time;
                if (elapsed < timeout_secs * 1000) {
                    setTimeout(attempt_nav_page, 100, page_name, start_time, timeout_secs);
                } else {
                    alert("Unable to navigate to page '" + page_name + "' after " + timeout_secs + " second(s).");
                }
            }
            window.addEventListener("load", function() {
                attempt_nav_page("%s", new Date(), %d);
            });
        </script>
    """ % (
        page_name,
        timeout_secs,
    )
    html(nav_script)



import streamlit as st
from Config import Config
from Agent import Agent
import os
import logging


def agent_selector():
    with st.sidebar:
        st.header("Select Agent")

        # Load the previously selected agent name
        try:
            with open(os.path.join("session.txt"), "r") as f:
                previously_selected_agent = f.read().strip()
        except FileNotFoundError:
            previously_selected_agent = None

        # Get the list of agent names
        agent_names = [agent["name"] for agent in Config().get_agents()]

        # If the previously selected agent is in the list, use it as the default
        if previously_selected_agent in agent_names:
            default_index = (
                agent_names.index(previously_selected_agent) + 1
            )  # add 1 for the empty string at index 0
        else:
            default_index = 0

        # Create the selectbox
        selected_agent = st.selectbox(
            "Agent Name",
            options=[""] + agent_names,
            index=default_index,
        )

        # If the selected agent has changed, save the new selection
        if selected_agent != previously_selected_agent:
            with open(os.path.join("session.txt"), "w") as f:
                f.write(selected_agent)
            try:
                st.experimental_rerun()
            except Exception as e:
                logging.info(e)
        if selected_agent != "":
            agent = Agent(selected_agent)
        else:
            agent = None
        return selected_agent, agent



from AGiXT import AGiXT
import re
import os
import json
import yaml
from Agent import Agent
from collections import deque
import logging


class Tasks:
    def __init__(self, agent_name: str = "AGiXT"):
        self.agent_name = agent_name
        self.ai = AGiXT(self.agent_name)
        self.primary_objective = None
        self.task_list = deque([])
        self.output_list = []
        self.stop_running_event = False
        if not os.path.exists(f"agents/{self.agent_name}/tasks"):
            os.makedirs(f"agents/{self.agent_name}/tasks")

    def load_task(self, task):
        out_file = re.sub(r"[^\w\s]", "", task)
        out_file = out_file[:25]
        try:
            with open(f"agents/{self.agent_name}/{out_file}.json", "r") as f:
                task_state = json.load(f)

            self.task_list = deque(task_state["task_list"])
            self.output_list = task_state["tasks"]
            self.primary_objective = task_state["primary_objective"]
            logging.info(f"Successfully loaded task '{out_file}'.")

        except FileNotFoundError:
            logging.info(f"No saved task found with the name '{out_file}'.")
        except Exception as e:
            logging.info(f"An error occurred while loading the task: {e}")

    def get_status(self):
        if self.task_list != deque([]) and self.output_list != []:
            return True
        else:
            return False

    def update_task(self, task_id, task_name, task_output):
        # Check if the task exists at agents/{agent_name}/tasks/{self.primary_objective}.json
        # We need to stip out symbols except spaces in primary objective and truncate the primary objective to 15 characters
        out_file = re.sub(r"[^\w\s]", "", self.primary_objective)
        out_file = out_file[:25]
        output_file = os.path.join(
            "agents", self.agent_name, "tasks", f"{out_file}.json"
        )
        if os.path.exists(output_file):
            # If it does, load it
            with open(output_file, "r") as f:
                data = json.load(f)
        else:
            # If it doesn't, create it
            data = {
                "primary_objective": self.primary_objective,
                "task_list": list(self.task_list),
                "tasks": [],
            }
        # Check if task is in the data["tasks"] list
        data["tasks"].append(
            {
                "task_id": task_id,
                "task_name": task_name,
                "task_output": task_output,
            }
        )
        # Save the data to agents/{agent_name}/tasks/{self.primary_objective}.json
        with open(output_file, "w") as f:
            json.dump(data, f)
        return data

    def get_task_output(self):
        out_file = re.sub(r"[^\w\s]", "", self.primary_objective)
        out_file = out_file[:25]
        try:
            with open(f"agents/{self.agent_name}/tasks/{out_file}.json", "r") as f:
                task_output = json.load(f)

            logging.info(f"Successfully loaded task output for '{out_file}'.")
            return task_output

        except FileNotFoundError:
            logging.info(f"No saved task output found with the name '{out_file}'.")
            return None
        except Exception as e:
            logging.info(f"An error occurred while loading the task output: {e}")
            return None

    def get_tasks_files(self):
        files = os.listdir(os.path.join("agents", self.agent_name, "tasks"))
        files = [file[:-5] for file in files]
        return files

    def stop_tasks(self):
        if self.stop_running_event is not None:
            self.stop_running_event = True
        self.task_list.clear()

    async def instruction_agent(self, task, **kwargs):
        if "task_name" in task:
            task = task["task_name"]

        resolver = await self.ai.run(
            task=task,
            prompt="SmartInstruct-StepByStep"
            if self.primary_objective is None
            else "SmartTask-StepByStep",
            context_results=6,
            objective=self.primary_objective,
            **kwargs,
        )
        # Check if agent has commands before trying to run execution agent

        if Agent(agent_name=self.agent_name).get_commands_string() != None:
            execution_response = await self.ai.run(
                task=task,
                prompt="SmartInstruct-Execution"
                if self.primary_objective is None
                else "SmartTask-Execution",
                previous_response=resolver,
                objective=self.primary_objective,
                **kwargs,
            )
            return f"RESPONSE:\n{resolver}\n\nCommand Execution Response{execution_response}"
        else:
            return f"RESPONSE:\n{resolver}"

    async def run_task(
        self,
        objective: str = "",
        smart: bool = False,
        load_task: str = "",
        **kwargs,
    ):
        initial_task = "Break down the objective into a list of small achievable tasks in the form of instructions that lead up to achieving the ultimate goal of the objective."
        if load_task != "":
            self.load_task(load_task)
            logging.info(f"Loaded task '{load_task}'.\n\n")
        else:
            self.primary_objective = objective
            self.task_list = deque(
                [
                    {
                        "task_id": 1,
                        "task_name": initial_task,
                    }
                ]
            )
            logging.info(f"Starting task with objective: {self.primary_objective}.\n\n")

        while not self.stop_running_event and self.task_list != deque([]):
            task = self.task_list.popleft()
            if task["task_name"] in ["None", "None.", ""]:
                self.stop_tasks()
                continue
            logging.info(f"\nExecuting task {task['task_id']}: {task['task_name']}\n")
            if smart != True:
                result = await self.instruction_agent(task=task["task_name"], **kwargs)
            else:
                result = await self.ai.smart_instruct(
                    task=task["task_name"],
                    shots=3,
                    objective=self.primary_objective,
                    **kwargs,
                )
            self.update_task(task["task_id"], task["task_name"], result)
            logging.info(f"\nTask Result:\n\n{result}\n")
            if task["task_name"] == initial_task:
                lines = result.split("\n") if "\n" in result else [result]
                new_tasks = []
                for line in lines:
                    match = re.match(r"(\d+)\.\s+(.*)", line)
                    if match:
                        task_id, task_name = match.groups()
                        new_tasks.append(
                            {"task_id": int(task_id), "task_name": task_name.strip()}
                        )
                self.task_list = deque(new_tasks)
        if not self.task_list:
            self.stop_tasks()
        logging.info("All tasks completed or stopped.")



import openai


class OpenaiProvider:
    def __init__(
        self,
        OPENAI_API_KEY: str = "",
        AI_MODEL: str = "gpt-3.5-turbo",
        AI_TEMPERATURE: float = 0.7,
        MAX_TOKENS: int = 4096,
        **kwargs
    ):
        self.requirements = ["openai"]
        self.AI_MODEL = AI_MODEL
        self.AI_TEMPERATURE = AI_TEMPERATURE
        self.MAX_TOKENS = MAX_TOKENS
        openai.api_key = OPENAI_API_KEY

    def instruct(self, prompt, tokens: int = 0):
        max_new_tokens = int(self.MAX_TOKENS) - tokens
        if not self.AI_MODEL.startswith("gpt-"):
            # Use completion API
            response = openai.Completion.create(
                engine=self.AI_MODEL,
                prompt=prompt,
                temperature=float(self.AI_TEMPERATURE),
                max_tokens=max_new_tokens,
                top_p=1,
                frequency_penalty=0,
                presence_penalty=0,
            )
            return response.choices[0].text.strip()
        else:
            # Use chat completion API
            messages = [{"role": "system", "content": prompt}]
            response = openai.ChatCompletion.create(
                model=self.AI_MODEL,
                messages=messages,
                temperature=float(self.AI_TEMPERATURE),
                max_tokens=max_new_tokens,
                n=1,
                stop=None,
            )
            return response.choices[0].message.content.strip()



import requests
import json
import os
from Extensions import Extensions
from io import BytesIO
import requests
from PIL import Image
import logging


class huggingface(Extensions):
    def __init__(
        self,
        HUGGINGFACE_API_KEY: str = "",
        HUGGINGFACE_AUDIO_TO_TEXT_MODEL: str = "facebook/wav2vec2-large-960h-lv60-self",
        WORKING_DIRECTORY: str = "./WORKSPACE",
        **kwargs,
    ):
        self.HUGGINGFACE_API_KEY = HUGGINGFACE_API_KEY
        self.HUGGINGFACE_AUDIO_TO_TEXT_MODEL = HUGGINGFACE_AUDIO_TO_TEXT_MODEL
        self.WORKING_DIRECTORY = WORKING_DIRECTORY
        if self.HUGGINGFACE_API_KEY is not None:
            self.commands = {
                "Read Audio from File": self.read_audio_from_file,
                "Read Audio": self.read_audio,
                "Generate Image with Stable Diffusion": self.generate_image_with_hf,
            }

    async def generate_image_with_hf(self, prompt: str, filename: str) -> str:
        API_URL = (
            "https://api-inference.huggingface.co/models/CompVis/stable-diffusion-v1-4"
        )
        headers = {"Authorization": f"Bearer {self.HUGGINGFACE_API_TOKEN}"}

        response = requests.post(
            API_URL,
            headers=headers,
            json={
                "inputs": prompt,
            },
        )

        image = Image.open(BytesIO(response.content))
        logging.info(f"Image Generated for prompt:{prompt}")

        image.save(os.path.join(self.WORKING_DIRECTORY, filename))

        return f"Saved to disk:{filename}"

    async def read_audio_from_file(self, audio_path: str):
        audio_path = os.path.join(self.WORKING_DIRECTORY, audio_path)
        with open(audio_path, "rb") as audio_file:
            audio = audio_file.read()
        return await self.read_audio(audio)

    async def read_audio(self, audio):
        model = self.HUGGINGFACE_AUDIO_TO_TEXT_MODEL
        api_url = f"https://api-inference.huggingface.co/models/{model}"
        api_token = self.HUGGINGFACE_API_KEY
        headers = {"Authorization": f"Bearer {api_token}"}

        if api_token is None:
            raise ValueError(
                "You need to set your Hugging Face API token in the config file."
            )

        response = requests.post(
            api_url,
            headers=headers,
            data=audio,
        )

        text = json.loads(response.content.decode("utf-8"))["text"]
        return "The audio says: " + text



from Extensions import Extensions
import os
import requests
from playsound import playsound


class briantts(Extensions):
    def __init__(
        self,
        USE_BRIAN_TTS: bool = True,
        **kwargs,
    ):
        self.USE_BRIAN_TTS = USE_BRIAN_TTS
        if self.USE_BRIAN_TTS:
            self.commands = {"Speak with TTS with BrianTTS": self.speak_with_briantts}

    async def speak_with_briantts(self, text: str) -> bool:
        tts_url = (
            f"https://api.streamelements.com/kappa/v2/speech?voice=Brian&text={text}"
        )
        response = requests.get(tts_url)

        if response.status_code == 200:
            with open("speech.mp3", "wb") as f:
                f.write(response.content)
            playsound("speech.mp3")
            os.remove("speech.mp3")
            return True
        else:
            return False



from time import time
import requests
import logging


class HuggingfaceProvider:
    def __init__(
        self,
        HUGGINGFACE_API_KEY: str = "",
        HUGGINGFACE_API_URL: str = "",
        AI_MODEL: str = "gpt2",
        AI_TEMPERATURE: float = 0.7,
        MAX_TOKENS: int = 4096,
        **kwargs,
    ):
        self.HUGGINGFACE_API_KEY = HUGGINGFACE_API_KEY
        self.HUGGINGFACE_API_URL = HUGGINGFACE_API_URL
        self.AI_MODEL = AI_MODEL
        self.AI_TEMPERATURE = AI_TEMPERATURE
        self.MAX_TOKENS = MAX_TOKENS

    def instruct(self, prompt: str, tokens: int = 0) -> str:
        num_retries = 3
        headers = {"Authorization": f"Bearer {self.HUGGINGFACE_API_KEY}"}
        payload = {
            "inputs": prompt,
            "max_tokens": int(self.MAX_TOKENS),
            "temperature": float(self.AI_TEMPERATURE),
        }
        for _ in range(num_retries):
            try:
                response = requests.post(
                    self.HUGGINGFACE_API_URL,
                    headers=headers,
                    json=payload,
                )
                response.raise_for_status()
                return response.json()[0]["generated_text"]
            except requests.exceptions.RequestException as e:
                logging.error(e)
                logging.info("Rate limit exceeded. Retrying after 20 seconds.")
                time.sleep(20)
                continue



from .AGiXT import AGiXT
from .Memories import Memories
from .Chain import Chain
from .Extensions import Extensions
from .Agent import Agent
from .Config import Config
from .Embedding import Embedding
from .Prompts import Prompts



from EdgeGPT import Chatbot, ConversationStyle
import asyncio
import os
import json


class BingProvider:
    def __init__(self, AI_TEMPERATURE: float = 0.7, **kwargs):
        self.AI_MODEL = "bing"
        self.requirements = ["EdgeGPT"]
        self.AI_TEMPERATURE = AI_TEMPERATURE

    async def ask(self, prompt, tokens: int = 0):
        cookie_path = "./cookies.json"
        # Check if cookies exist
        if os.path.exists(cookie_path):
            with open(cookie_path, "r") as f:
                cookies = f.read()
            cookies = json.loads(cookies)
        try:
            bot = await Chatbot.create(cookies=cookies)
            response = await bot.ask(
                prompt=prompt,
                conversation_style=ConversationStyle.creative,
            )
            await bot.close()
            return response
        except Exception as e:
            return f"EdgeGPT Error: {e}"

    def instruct(self, prompt, tokens: int = 0):
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)

        async def _ask_and_set_result():
            result = await self.ask(prompt, tokens)
            return result

        response = loop.run_until_complete(_ask_and_set_result())

        # Closing the loop after use
        loop.close()

        # Extract the text of the bot's message
        bot_response = ""
        for message in response.get("item", {}).get("messages", []):
            if message.get("author") == "bot":
                bot_response = message.get("text") or message.get("hiddenText")

                # If the response is in an Adaptive Card
                if "adaptiveCards" in message:
                    for card in message["adaptiveCards"]:
                        for item in card.get("body", []):
                            if item["type"] == "TextBlock":
                                bot_response = item.get("text")

                # Stop at the first bot message we find
                if bot_response:
                    break

        return bot_response



import tweepy
from Extensions import Extensions
import logging


class twitter(Extensions):
    def __init__(
        self,
        TW_CONSUMER_KEY: str = "",
        TW_CONSUMER_SECRET: str = "",
        TW_ACCESS_TOKEN: str = "",
        TW_ACCESS_TOKEN_SECRET: str = "",
        **kwargs
    ):
        self.TW_CONSUMER_KEY = TW_CONSUMER_KEY
        self.TW_CONSUMER_SECRET = TW_CONSUMER_SECRET
        self.TW_ACCESS_TOKEN = TW_ACCESS_TOKEN
        self.TW_ACCESS_TOKEN_SECRET = TW_ACCESS_TOKEN_SECRET
        if self.TW_CONSUMER_KEY and self.TW_ACCESS_TOKEN:
            self.commands = {"Send Tweet": self.send_tweet}

    async def send_tweet(self, tweet_text):
        # Authenticate to Twitter
        auth = tweepy.OAuthHandler(self.TW_CONSUMER_KEY, self.TW_CONSUMER_SECRET)
        auth.set_access_token(self.TW_ACCESS_TOKEN, self.TW_ACCESS_TOKEN_SECRET)

        # Create API object
        api = tweepy.API(auth)

        # Send tweet
        try:
            api.update_status(tweet_text)
            logging.info("Tweet sent successfully!")
        except tweepy.TweepyException as e:
            logging.info("Error sending tweet: {}".format(e.reason))



from typing import Union, List
import json
from Extensions import Extensions


class google(Extensions):
    def __init__(self, GOOGLE_API_KEY: str = "", **kwargs):
        self.GOOGLE_API_KEY = GOOGLE_API_KEY
        if self.GOOGLE_API_KEY:
            self.commands = {
                "Google Search": self.google_official_search,
            }

    async def google_official_search(
        self, query: str, num_results: int = 8
    ) -> Union[str, List[str]]:
        from googleapiclient.discovery import build
        from googleapiclient.errors import HttpError

        try:
            # Get the Google API key and Custom Search Engine ID from the config file
            api_key = self.GOOGLE_API_KEY
            custom_search_engine_id = self.custom_search_engine_id

            # Initialize the Custom Search API service
            service = build("customsearch", "v1", developerKey=api_key)

            # Send the search query and retrieve the results
            result = (
                service.cse()
                .list(q=query, cx=custom_search_engine_id, num=num_results)
                .execute()
            )

            # Extract the search result items from the response
            search_results = result.get("items", [])

            # Create a list of only the URLs from the search results
            search_results_links = [item["link"] for item in search_results]

        except HttpError as e:
            # Handle errors in the API call
            error_details = json.loads(e.content.decode())

            # Check if the error is related to an invalid or missing API key
            if error_details.get("error", {}).get(
                "code"
            ) == 403 and "invalid API key" in error_details.get("error", {}).get(
                "message", ""
            ):
                return "Error: The provided Google API key is invalid or missing."
            else:
                return f"Error: {e}"

        # Return the list of search result URLs
        return search_results_links



import streamlit as st
from Prompts import Prompts
from auth_libs.Users import check_auth_status

check_auth_status()

st.header("Manage Prompts")
st.markdown("### Usage Instructions")
st.markdown(
    """
To create dynamic prompts that can have user inputs, you can use curly braces `{}` in your prompt content. 
Anything between the curly braces will be considered as an input field. For example:

```python
"Hello, my name is {name} and I'm {age} years old."
```
In the above prompt, `name` and `age` will be the input arguments. These arguments can be used in chains.
"""
)
st.markdown("### Predefined Injection Variables")
st.markdown(
    """
- `{agent_name}` will cause the agent name to be injected.
- `{context}` will cause the current context from memory to be injected.
- `{date}` will cause the current date and timestamp to be injected.
- `{COMMANDS}` will cause the available commands list to be injected and for automatic commands execution from the agent based on its suggestions.
- `{command_list}` will cause the available commands list to be injected, but will not execute any commands the AI chooses. Useful on validation steps.
- `{STEPx}` will cause the step `x` response from a chain to be injected. For example, `{STEP1}` will inject the first step's response in a chain.
"""
)

custom_prompt = Prompts()
prompt_list = custom_prompt.get_prompts()

if st.checkbox("Add New Prompt"):
    action = "Add Prompt"
    prompt_name = st.text_input("Prompt Name")
    prompt_content = st.text_area("Prompt Content", height=300)
else:
    action = st.selectbox("Action", ["Update Prompt", "Delete Prompt"])
    prompt_name = st.selectbox("Existing Prompts", prompt_list)
    prompt_content = st.text_area(
        "Prompt Content",
        custom_prompt.get_prompt(prompt_name) if prompt_name else "",
        height=300,
    )

if st.button("Perform Action"):
    if prompt_name and (prompt_content or action == "Delete Prompt"):
        if action == "Add Prompt":
            custom_prompt.add_prompt(prompt_name, prompt_content)
            st.success(f"Prompt '{prompt_name}' added.")
        elif action == "Update Prompt":
            custom_prompt.update_prompt(prompt_name, prompt_content)
            st.success(f"Prompt '{prompt_name}' updated.")
        elif action == "Delete Prompt":
            custom_prompt.delete_prompt(prompt_name)
            st.success(f"Prompt '{prompt_name}' deleted.")
    else:
        st.error("Prompt name and content are required.")



import subprocess
import sys

try:
    from nomic.gpt4all import GPT4AllGPU as GPT4All
except ImportError:
    subprocess.check_call([sys.executable, "-m", "pip", "install", "nomic"])
    from nomic.gpt4all import GPT4AllGPU as GPT4All


class Gpugpt4allProvider:
    def __init__(
        self,
        MODEL_PATH: str = "",
        MAX_TOKENS: int = 2000,
        AI_MODEL: str = "default",
        AI_TEMPERATURE: float = 0.7,
        **kwargs,
    ):
        try:
            self.max_tokens = int(MAX_TOKENS)
        except:
            self.max_tokens = 2000
        self.AI_MODEL = AI_MODEL
        self.AI_TEMPERATURE = AI_TEMPERATURE
        # GPT4All will just download the model, maybe save it in our workspace?
        self.model = GPT4All(llama_path=MODEL_PATH)
        self.config = {
            "num_beams": 2,
            "min_new_tokens": 10,
            "max_length": 100,
            "repetition_penalty": 2.0,
        }
        # TODO: Need to reseach to add temperature, no obvious flag.

    def instruct(self, prompt):
        try:
            return self.model.generate(prompt, self.config)
        except Exception as e:
            return f"GPT4ALL Error: {e}"



import streamlit as st
from AGiXT import AGiXT
from streamlit import (
    markdown,
    header,
    checkbox,
    container,
    text_input,
    button,
    spinner,
    error,
    warning,
)
import asyncio
from auth_libs.Users import check_auth_status
from components.agent_selector import agent_selector

check_auth_status()
agent_name, agent = agent_selector()


def render_history(instruct_container, chat_history):
    instruct_container.empty()
    with instruct_container:
        for instruct in chat_history:
            if "sender" in instruct and "message" in instruct:
                if instruct["sender"] == "User":
                    markdown(
                        f'<div style="text-align: left; margin-bottom: 5px;"><strong>User:</strong> {instruct["message"]}</div>',
                        unsafe_allow_html=True,
                    )
                else:
                    markdown(
                        f'<div style="text-align: left; margin-bottom: 5px;"><strong>Agent:</strong> {instruct["message"]}</div>',
                        unsafe_allow_html=True,
                    )


header("Instruct an Agent")

smart_instruct_toggle = checkbox("Enable Smart Instruct")

if "chat_history" not in st.session_state:
    st.session_state["chat_history"] = {}

instruct_container = container()

if agent_name:
    try:
        st.session_state.chat_history[agent_name] = agent.get_chat_history(agent_name)
    except:
        st.session_state.chat_history[
            agent_name
        ] = []  # initialize as an empty list, not a dictionary

    render_history(instruct_container, st.session_state.chat_history[agent_name])

    instruct_prompt = text_input("Enter your message", key="instruct_prompt")
    send_button = button("Send Message")

    if send_button:
        if agent_name and instruct_prompt:
            with spinner("Thinking, please wait..."):
                agent = AGiXT(agent_name)
                if smart_instruct_toggle:
                    response = asyncio.run(
                        agent.smart_instruct(
                            instruct_prompt,
                            shots=3,
                        )
                    )
                else:
                    response = asyncio.run(
                        agent.run(
                            instruct_prompt,
                            prompt="instruct",
                            context_results=6,
                        )
                    )
            instruct_entry = [
                {"sender": "User", "message": instruct_prompt},
                {"sender": "Agent", "message": response},
            ]
            st.session_state.chat_history[agent_name].extend(instruct_entry)
            render_history(
                instruct_container,
                st.session_state.chat_history[agent_name],
            )
        else:
            error("Agent name and message are required.")
else:
    warning("Please select an agent to start instructting.")



import os
import glob
import logging

logging.basicConfig(
    level=os.environ.get("LOGLEVEL", "INFO"),
    format="%(asctime)s | %(levelname)s | %(message)s",
)


class Config:
    def get_providers(self):
        providers = []
        for provider in glob.glob("provider/*.py"):
            if "__init__.py" not in provider:
                providers.append(os.path.splitext(os.path.basename(provider))[0])
        return providers

    def get_agents(self):
        memories_dir = "agents"
        if not os.path.exists(memories_dir):
            os.makedirs(memories_dir)
        agents = []
        for file in os.listdir(memories_dir):
            if file.endswith(".yaml"):
                agents.append(file.replace(".yaml", ""))
        output = []
        if agents:
            for agent in agents:
                try:
                    agent_instance = self.agent_instances[agent]
                    status = agent_instance.get_status()
                except:
                    status = False
                output.append({"name": agent, "status": status})
        return output



from base64 import b64decode
import openai
from Extensions import Extensions
import logging


class dalle(Extensions):
    def __init__(
        self,
        HUGGINGFACE_API_KEY: str = "",
        OPENAI_API_KEY: str = "",
        WORKING_DIRECTORY: str = "./WORKSPACE",
        **kwargs,
    ):
        self.HUGGINGFACE_API_KEY = HUGGINGFACE_API_KEY
        self.OPENAI_API_KEY = OPENAI_API_KEY
        self.WORKING_DIRECTORY = WORKING_DIRECTORY
        if self.OPENAI_API_KEY:
            self.commands = {
                "Generate Image with DALLE": self.generate_image_with_dalle
            }

    async def generate_image_with_dalle(self, prompt: str, filename: str) -> str:
        openai.api_key = self.OPENAI_API_KEY

        response = openai.Image.create(
            prompt=prompt,
            n=1,
            size="256x256",
            response_format="b64_json",
        )

        logging.info(f"Image Generated for prompt:{prompt}")

        image_data = b64decode(response["data"][0]["b64_json"])

        with open(f"{self.WORKING_DIRECTORY}/{filename}", mode="wb") as png:
            png.write(image_data)

        return f"Saved to disk:{filename}"



# pip install discord.py
import discord as dc
from discord.ext import commands
from Extensions import Extensions
import logging


class discord(Extensions):
    def __init__(
        self,
        DISCORD_API_KEY: str = "",
        DISCORD_COMMAND_PREFIX: str = "/AGiXT",
        **kwargs,
    ):
        self.DISCORD_API_KEY = DISCORD_API_KEY
        self.DISCORD_COMMAND_PREFIX = DISCORD_COMMAND_PREFIX
        if self.DISCORD_API_KEY:
            self.commands = {
                "Send Discord Message": self.send_message,
                "Get Discord Messages": self.get_messages,
                "Delete Discord Message": self.delete_message,
                "Create Discord Invite": self.create_invite,
                "Get Discord Servers": self.get_servers,
                "Get Discord Server Information": self.get_server_info,
            }

            try:
                intents = dc.Intents.default()
                intents.typing = False
                intents.presences = False
                if self.DISCORD_API_KEY != "None" and self.DISCORD_API_KEY != "":
                    self.bot = commands.Bot(
                        command_prefix=self.DISCORD_COMMAND_PREFIX, intents=intents
                    )
                    self.bot.run(self.DISCORD_API_KEY)

                @self.bot.event
                async def on_ready():
                    logging.info(f"{self.bot.user.name} is ready")

                @self.bot.event
                async def on_command_error(ctx, error):
                    await ctx.send(f"Error: {error}")

            except:
                pass

    async def send_message(self, channel_id: int, content: str):
        channel = self.bot.get_channel(channel_id)
        await channel.send(content)

    async def get_messages(self, channel_id: int, limit: int = 100):
        channel = self.bot.get_channel(channel_id)
        messages = await channel.history(limit=limit).flatten()
        return [(message.author, message.content) for message in messages]

    async def delete_message(self, channel_id: int, message_id: int):
        channel = self.bot.get_channel(channel_id)
        message = await channel.fetch_message(message_id)
        await message.delete()

    async def create_invite(self, channel_id: int, max_age: int = 0, max_uses: int = 0):
        channel = self.bot.get_channel(channel_id)
        invite = await channel.create_invite(max_age=max_age, max_uses=max_uses)
        return invite.url

    async def get_servers(self):
        return [guild.name for guild in self.bot.guilds]

    async def get_server_info(self, server_id: int):
        server = self.bot.get_guild(server_id)
        return {
            "name": server.name,
            "owner": server.owner,
            "member_count": server.member_count,
        }



from typing import List, Union
from requests.compat import urljoin
from bs4 import BeautifulSoup
from Extensions import Extensions
from playwright.sync_api import sync_playwright


class web_playwright(Extensions):
    def __init__(self, **kwargs):
        self.commands = {
            "Scrape Text with Playwright": self.scrape_text_with_playwright,
            "Scrape Links with Playwright": self.scrape_links_with_playwright,
        }

    async def scrape_text_with_playwright(self, url: str) -> str:
        with sync_playwright() as p:
            browser = p.chromium.launch()
            page = browser.new_page()

            try:
                page.goto(url)
                html_content = page.content()
                soup = BeautifulSoup(html_content, "html.parser")

                for script in soup(["script", "style"]):
                    script.extract()

                text = soup.get_text()
                lines = (line.strip() for line in text.splitlines())
                chunks = (
                    phrase.strip() for line in lines for phrase in line.split("  ")
                )
                text = "\n".join(chunk for chunk in chunks if chunk)

            except Exception as e:
                text = f"Error: {str(e)}"

            finally:
                browser.close()

        return text

    async def scrape_links_with_playwright(self, url: str) -> Union[str, List[str]]:
        with sync_playwright() as p:
            browser = p.chromium.launch()
            page = browser.new_page()

            try:
                page.goto(url)
                html_content = page.content()
                soup = BeautifulSoup(html_content, "html.parser")

                for script in soup(["script", "style"]):
                    script.extract()

                hyperlinks = [
                    (link.text, urljoin(url, link["href"]))
                    for link in soup.find_all("a", href=True)
                ]
                formatted_links = [
                    f"{link_text} ({link_url})" for link_text, link_url in hyperlinks
                ]

            except Exception as e:
                formatted_links = f"Error: {str(e)}"

            finally:
                browser.close()

        return formatted_links



from Extensions import Extensions
import os


class macostts(Extensions):
    def __init__(
        self,
        USE_MAC_OS_TTS: bool = False,
        **kwargs,
    ):
        self.USE_MAC_OS_TTS = USE_MAC_OS_TTS
        self.commands = {"Speak with MacOS TTS": self.speak_with_macos_speech}

    async def speak_with_macos_speech(self, text: str, voice_index: int = 0) -> bool:
        if voice_index == 0:
            os.system(f'say "{text}"')
        elif voice_index == 1:
            os.system(f'say -v "Ava (Premium)" "{text}"')
        else:
            os.system(f'say -v Samantha "{text}"')
        return True



import re
import os
import regex
import json
import time
import spacy
from datetime import datetime
from Agent import Agent
from Prompts import Prompts
from extensions.searxng import searxng
from urllib.parse import urlparse
import logging
from concurrent.futures import Future


class AGiXT:
    def __init__(self, agent_name: str = "AGiXT"):
        self.agent_name = agent_name
        self.agent = Agent(self.agent_name)
        self.agent_commands = self.agent.get_commands_string()
        self.stop_running_event = None
        self.browsed_links = []
        self.failures = 0
        self.nlp = None

    def load_spacy_model(self):
        if not self.nlp:
            try:
                self.nlp = spacy.load("en_core_web_sm")
            except:
                spacy.cli.download("en_core_web_sm")
                self.nlp = spacy.load("en_core_web_sm")
        self.nlp.max_length = 99999999999999999999999

    def custom_format(self, string, **kwargs):
        if isinstance(string, list):
            string = "".join(str(x) for x in string)

        def replace(match):
            key = match.group(1)
            value = kwargs.get(key, match.group(0))
            if isinstance(value, list):
                return "".join(str(x) for x in value)
            else:
                return str(value)

        pattern = r"(?<!{){([^{}\n]+)}(?!})"
        result = re.sub(pattern, replace, string)
        return result

    def get_step_response(self, chain_name, step_number):
        try:
            with open(os.path.join("chains", f"{chain_name}_responses.json"), "r") as f:
                responses = json.load(f)
            return responses.get(str(step_number))
        except:
            return ""

    async def format_prompt(
        self,
        task: str = "",
        top_results: int = 5,
        prompt="",
        chain_name="",
        step_number=0,
        memories=None,
        **kwargs,
    ):
        cp = Prompts()
        if prompt == "":
            prompt = task
        else:
            try:
                prompt = cp.get_prompt(prompt_name=prompt, model=self.agent.AI_MODEL)
            except:
                prompt = prompt
        if top_results == 0:
            context = "None"
        else:
            try:
                context = await memories.context_agent(
                    query=task, top_results_num=top_results
                )
            except:
                context = "None."
        command_list = self.agent.get_commands_string()
        formatted_prompt = self.custom_format(
            prompt,
            task=task,
            agent_name=self.agent_name,
            COMMANDS=self.agent_commands,
            context=context,
            command_list=command_list,
            date=datetime.now().strftime("%B %d, %Y %I:%M %p"),
            **kwargs,
        )
        if "{STEP" in formatted_prompt:
            # get the response from the step number
            step_response = self.get_step_response(
                chain_name=chain_name, step_number=step_number
            )
            # replace the {STEPx} with the response
            formatted_prompt = formatted_prompt.replace(
                f"{{STEP{step_number}}}", step_response
            )
        if not self.nlp:
            self.load_spacy_model()
        tokens = len(self.nlp(formatted_prompt))
        logging.info(f"FORMATTED PROMPT: {formatted_prompt}")
        return formatted_prompt, prompt, tokens

    async def run(
        self,
        task: str = "",
        prompt: str = "",
        context_results: int = 5,
        websearch: bool = False,
        websearch_depth: int = 3,
        learn_file: str = "",
        chain_name: str = "",
        step_number: int = 0,
        **kwargs,
    ):
        logging.info(f"KWARGS: {kwargs}")
        memories = self.agent.get_memories()
        if learn_file != "":
            learning_file = await memories.mem_read_file(file_path=learn_file)
            if learning_file == False:
                return "Failed to read file."
        formatted_prompt, unformatted_prompt, tokens = await self.format_prompt(
            task=task,
            top_results=context_results,
            prompt=prompt,
            chain_name=chain_name,
            step_number=step_number,
            memories=memories,
            **kwargs,
        )
        if websearch:
            await self.websearch_agent(task=task, depth=websearch_depth)
        try:
            # Workaround for non-threaded providers
            run_response = self.agent.instruct(formatted_prompt, tokens=tokens)
            self.response = (
                run_response.result()
                if isinstance(run_response, Future)
                else run_response
            )
        except Exception as e:
            logging.info(f"Error: {e}")
            logging.info(f"PROMPT CONTENT: {formatted_prompt}")
            logging.info(f"TOKENS: {tokens}")
            self.failures += 1
            if self.failures == 5:
                self.failures == 0
                logging.info("Failed to get a response 5 times in a row.")
                return None
            logging.info(f"Retrying in 10 seconds...")
            time.sleep(10)
            if context_results > 0:
                context_results = context_results - 1
            self.response = await self.run(
                task=task,
                prompt=prompt,
                context_results=context_results,
                **kwargs,
            )

        # Handle commands if the prompt contains the {COMMANDS} placeholder
        # We handle command injection that DOESN'T allow command execution by using {command_list} in the prompt
        if "{COMMANDS}" in unformatted_prompt:
            execution_response = await self.execution_agent(
                execution_response=self.response,
                task=task,
                context_results=context_results,
                **kwargs,
            )
            return_response = ""
            try:
                self.response = json.loads(self.response)
                if "response" in self.response:
                    return_response = self.response["response"]
                if "commands" in self.response:
                    if self.response["commands"] != {}:
                        return_response += (
                            f"\n\nCommands Executed:\n{self.response['commands']}"
                        )
                if execution_response:
                    return_response += (
                        f"\n\nCommand Execution Response:\n{execution_response}"
                    )
            except:
                return_response = self.response
            self.response = return_response
        logging.info(f"Response: {self.response}")
        if self.response != "" and self.response != None:
            try:
                await memories.store_result(task, self.response)
            except:
                pass
            self.agent.log_interaction("USER", task)
            self.agent.log_interaction(self.agent_name, self.response)
        return self.response

    async def smart_instruct(
        self,
        task: str = "Write a tweet about AI.",
        shots: int = 3,
        learn_file: str = "",
        objective: str = None,
        **kwargs,
    ):
        answers = []
        # Do multi shots of prompt to get N different answers to be validated
        answers.append(
            await self.run(
                task=task,
                prompt="SmartInstruct-StepByStep"
                if objective == None
                else "SmartTask-StepByStep",
                context_results=6,
                websearch=True,
                websearch_depth=3,
                shots=shots,
                learn_file=learn_file,
                objective=objective,
                **kwargs,
            )
        )
        if shots > 1:
            for i in range(shots - 1):
                answers.append(
                    await self.run(
                        task=task,
                        prompt="SmartInstruct-StepByStep"
                        if objective == None
                        else "SmartTask-StepByStep",
                        context_results=6,
                        shots=shots,
                        objective=objective,
                        **kwargs,
                    )
                )
        answer_str = ""
        for i, answer in enumerate(answers):
            answer_str += f"Answer {i + 1}:\n{answer}\n\n"
        researcher = await self.run(
            task=answer_str,
            prompt="SmartInstruct-Researcher",
            shots=shots,
            **kwargs,
        )
        resolver = await self.run(
            task=researcher,
            prompt="SmartInstruct-Resolver",
            shots=shots,
            **kwargs,
        )
        execution_response = await self.run(
            task=task,
            prompt="SmartInstruct-Execution",
            previous_response=resolver,
            **kwargs,
        )
        clean_response_agent = await self.run(
            task=task,
            prompt="SmartInstruct-CleanResponse"
            if objective == None
            else "SmartTask-CleanResponse",
            resolver_response=resolver,
            execution_response=execution_response,
            objective=objective,
            **kwargs,
        )
        return clean_response_agent

    async def smart_chat(
        self,
        task: str = "Write a tweet about AI.",
        shots: int = 3,
        learn_file: str = "",
        **kwargs,
    ):
        answers = []
        answers.append(
            await self.run(
                task=task,
                prompt="SmartChat-StepByStep",
                context_results=6,
                websearch=True,
                websearch_depth=3,
                shots=shots,
                learn_file=learn_file,
                **kwargs,
            )
        )
        # Do multi shots of prompt to get N different answers to be validated
        if shots > 1:
            for i in range(shots - 1):
                answers.append(
                    await self.run(
                        task=task,
                        prompt="SmartChat-StepByStep",
                        context_results=6,
                        shots=shots,
                        **kwargs,
                    )
                )
        answer_str = ""
        for i, answer in enumerate(answers):
            answer_str += f"Answer {i + 1}:\n{answer}\n\n"
        researcher = await self.run(
            task=answer_str,
            prompt="SmartChat-Researcher",
            context_results=6,
            shots=shots,
            **kwargs,
        )
        resolver = await self.run(
            task=researcher,
            prompt="SmartChat-Resolver",
            context_results=6,
            shots=shots,
            **kwargs,
        )
        clean_response_agent = await self.run(
            task=task,
            prompt="SmartChat-CleanResponse",
            resolver_response=resolver,
            **kwargs,
        )
        return clean_response_agent

    # Worker Sub-Agents
    async def validation_agent(
        self, task, execution_response, context_results, **kwargs
    ):
        try:
            pattern = regex.compile(r"\{(?:[^{}]|(?R))*\}")
            cleaned_json = pattern.findall(execution_response)
            if len(cleaned_json) == 0:
                return False
            if isinstance(cleaned_json, list):
                cleaned_json = cleaned_json[0]
            response = json.loads(cleaned_json)
            return response
        except:
            logging.info("INVALID JSON RESPONSE")
            logging.info(execution_response)
            logging.info("... Trying again.")
            if context_results != 0:
                context_results = context_results - 1
            else:
                context_results = 0
            execution_response = await self.run(
                task=task, context_results=context_results, **kwargs
            )
            return await self.validation_agent(
                task, execution_response, context_results, **kwargs
            )

    async def revalidation_agent(
        self,
        task,
        command_name,
        command_args,
        command_output,
        context_results,
        **kwargs,
    ):
        logging.info(
            f"Command {command_name} did not execute as expected with args {command_args}. Trying again.."
        )
        revalidate = await self.run(
            task=task,
            prompt="ValidationFailed",
            command_name=command_name,
            command_args=command_args,
            command_output=command_output,
            **kwargs,
        )
        return await self.execution_agent(revalidate, task, context_results, **kwargs)

    async def execution_agent(
        self, execution_response, task, context_results, **kwargs
    ):
        validated_response = await self.validation_agent(
            task, execution_response, context_results, **kwargs
        )
        try:
            for command_name, command_args in validated_response["commands"].items():
                # Search for the command in the available_commands list, and if found, use the command's name attribute for execution
                if command_name is not None:
                    for available_command in self.agent.available_commands:
                        if command_name in [
                            available_command["friendly_name"],
                            available_command["name"],
                        ]:
                            command_name = available_command["friendly_name"]
                            try:
                                # Check if the command is a valid command in the self.avent.available_commands list
                                command_output = await self.agent.execute(
                                    command_name, command_args
                                )
                                logging.info("Running Command Execution Validation...")
                                validate_command = await self.run(
                                    task=task,
                                    prompt="Validation",
                                    command_name=command_name,
                                    command_args=command_args,
                                    command_output=command_output,
                                    **kwargs,
                                )
                                if validate_command.startswith("Yes"):
                                    # Failed command execution
                                    return await self.revalidation_agent(
                                        task=task,
                                        command_name=command_name,
                                        command_args=command_args,
                                        command_output=command_output,
                                        context_results=context_results,
                                        **kwargs,
                                    )
                                else:
                                    # Successful command execution
                                    logging.info(
                                        f"Command {command_name} executed successfully with args {command_args}. Command Output: {command_output}"
                                    )
                                    response = f"\nExecuted Command:{command_name} with args {command_args}.\nCommand Output: {command_output}\n"
                                    return response
                            except:
                                return await self.revalidation_agent(
                                    task=task,
                                    command_name=command_name,
                                    command_args=command_args,
                                    command_output=command_output,
                                    context_results=context_results,
                                    **kwargs,
                                )
                else:
                    if command_name == "None.":
                        return "\nNo commands were executed.\n"
                    else:
                        return f"\Command not recognized: {command_name} ."
        except:
            logging.info("\nERROR IN EXECUTION_AGENT, validated_response:\n")
            logging.info(validated_response)
            return "\nNo commands were executed.\n"

    async def websearch_agent(
        self, task: str = "What are the latest breakthroughs in AI?", depth: int = 3
    ):
        memories = self.agent.get_memories()

        async def resursive_browsing(task, links):
            try:
                words = links.split()
                links = [
                    word for word in words if urlparse(word).scheme in ["http", "https"]
                ]
            except:
                links = links
            if links is not None:
                for link in links:
                    if "href" in link:
                        try:
                            url = link["href"]
                        except:
                            url = link
                    else:
                        url = link
                    url = re.sub(r"^.*?(http)", r"http", url)
                    # Check if url is an actual url
                    if url.startswith("http"):
                        logging.info(f"Scraping: {url}")
                        if url not in self.browsed_links:
                            self.browsed_links.append(url)
                            (
                                collected_data,
                                link_list,
                            ) = await memories.read_website(url)
                            if link_list is not None:
                                if len(link_list) > 0:
                                    if len(link_list) > 5:
                                        link_list = link_list[:3]
                                    try:
                                        pick_a_link = await self.run(
                                            task=task,
                                            prompt="Pick-a-Link",
                                            links=link_list,
                                        )
                                        if not pick_a_link.startswith("None"):
                                            await resursive_browsing(task, pick_a_link)
                                    except:
                                        logging.info(
                                            f"Issues reading {url}. Moving on..."
                                        )

        results = await self.run(task=task, prompt="WebSearch")
        results = results.split("\n")
        for result in results:
            search_string = result.lstrip("0123456789. ")
            try:
                searx_server = self.agent.PROVIDER_SETTINGS["SEARXNG_INSTANCE_URL"]
            except:
                searx_server = ""
            try:
                links = searxng(SEARXNG_INSTANCE_URL=searx_server).search(search_string)
                if len(links) > depth:
                    links = links[:depth]
            except:
                links = None
            if links is not None:
                await resursive_browsing(task, links)



import os
import streamlit as st
from Config import Config
from Agent import Agent
from Extensions import Extensions
from Embedding import get_embedding_providers
from provider import get_provider_options
from auth_libs.Users import check_auth_status
from components.agent_selector import agent_selector

check_auth_status()

agent_name, agent = agent_selector()
CFG = Config()


def render_provider_settings(agent_settings, provider_name: str):
    try:
        required_settings = get_provider_options(provider_name)
    except (TypeError, ValueError):
        st.error(
            f"Error loading provider settings: expected a list, but got {required_settings}"
        )
        return {}
    rendered_settings = {}

    if not isinstance(required_settings, list):
        st.error(
            f"Error loading provider settings: expected a list, but got {required_settings}"
        )
        return rendered_settings

    for key in required_settings:
        if key in agent_settings:
            default_value = agent_settings[key]
        else:
            default_value = ""

        user_val = st.text_input(key, value=default_value)
        rendered_settings[key] = user_val

    if "LOG_REQUESTS" in agent_settings:
        value = agent_settings["LOG_REQUESTS"]
    else:
        value = False
    rendered_settings["LOG_REQUESTS"] = st.checkbox(
        "Log requests to files", key="LOG_REQUESTS", value=value
    )

    return rendered_settings


st.header("Manage Agent Settings")

if "new_agent_name" not in st.session_state:
    st.session_state.new_agent_name = ""

# Check if a new agent has been added and reset the session state variable
if st.session_state.new_agent_name and st.session_state.new_agent_name != agent_name:
    st.session_state.new_agent_name = ""

# Add an input field for the new agent's name
new_agent = False
if not agent_name:
    new_agent_name = st.text_input("New Agent Name")

    # Add an "Add Agent" button
    add_agent_button = st.button("Add Agent")

    # If the "Add Agent" button is clicked, create a new agent config file
    if add_agent_button:
        if new_agent_name:
            try:
                Agent(new_agent_name).add_agent(new_agent_name, {})
                st.success(f"Agent '{new_agent_name}' added.")
                agent_name = new_agent_name
                with open(os.path.join("session.txt"), "w") as f:
                    f.write(agent_name)
                st.session_state.new_agent_name = agent_name
                st.experimental_rerun()  # Rerun the app to update the agent list
            except Exception as e:
                st.error(f"Error adding agent: {str(e)}")
        else:
            st.error("New agent name is required.")
    new_agent = True

if agent_name and not new_agent:
    try:
        agent_config = agent.get_agent_config()
        agent_settings = agent_config.get("settings", {})
        provider_name = agent_settings.get("provider", "")
        provider_name = st.selectbox(
            "Select Provider",
            CFG.get_providers(),
            index=CFG.get_providers().index(provider_name)
            if provider_name in CFG.get_providers()
            else 0,
        )

        agent_settings[
            "provider"
        ] = provider_name  # Update the agent_settings with the selected provider

        embedder_name = agent_settings.get("embedder", "")
        embedders = get_embedding_providers()
        embedder_name = st.selectbox(
            "Select Embedder",
            embedders,
            index=embedders.index(embedder_name) if embedder_name in embedders else 0,
        )

        agent_settings[
            "embedder"
        ] = embedder_name  # Update the agent_settings with the selected embedder

        if provider_name:
            provider_settings = render_provider_settings(agent_settings, provider_name)
            agent_settings.update(provider_settings)

        def render_extension_settings(extension_settings, agent_settings):
            rendered_settings = {}

            for extension, settings in extension_settings.items():
                st.subheader(f"{extension}")
                for key, val in settings.items():
                    if key in agent_settings:
                        default_value = agent_settings[key]
                    else:
                        default_value = val if val else ""

                    user_val = st.text_input(
                        key, value=default_value, key=f"{extension}_{key}"
                    )

                    # Check if the user value exists before saving the setting
                    if user_val:
                        rendered_settings[key] = user_val

            return rendered_settings

        st.subheader("Extension Settings")
        extension_setting_keys = Extensions(agent_config).get_extension_settings()
        extension_settings = render_extension_settings(
            extension_setting_keys, agent_settings
        )

        # Update the extension settings in the agent_settings directly
        agent_settings.update(extension_settings)

        st.subheader("Custom Settings")
        custom_settings = agent_settings.get("custom_settings", [])

        custom_settings_list = st.session_state.get("custom_settings_list", None)
        if custom_settings_list is None:
            if not custom_settings:
                custom_settings = [""]
            st.session_state.custom_settings_list = custom_settings.copy()

        custom_settings_container = st.container()
        with custom_settings_container:
            for i, custom_setting in enumerate(st.session_state.custom_settings_list):
                key, value = (
                    custom_setting.split(":", 1)
                    if ":" in custom_setting
                    else (custom_setting, "")
                )
                col1, col2 = st.columns(
                    [0.5, 0.5]
                )  # Add columns for side by side input
                with col1:
                    new_key = st.text_input(
                        f"Custom Setting {i + 1} Key",
                        value=key,
                        key=f"custom_key_{i}",
                    )
                with col2:
                    new_value = st.text_input(
                        f"Custom Setting {i + 1} Value",
                        value=value,
                        key=f"custom_value_{i}",
                    )
                st.session_state.custom_settings_list[i] = f"{new_key}:{new_value}"

                # Automatically add an empty key/value pair if the last one is filled
                if (
                    i == len(st.session_state.custom_settings_list) - 1
                    and new_key
                    and new_value
                ):
                    st.session_state.custom_settings_list.append("")

        # Update the custom settings in the agent_settings directly
        agent_settings.update(
            {
                custom_setting.split(":", 1)[0]: custom_setting.split(":", 1)[1]
                for custom_setting in st.session_state.custom_settings_list
                if custom_setting and ":" in custom_setting
            }
        )

        st.subheader("Agent Commands")
        # Fetch the available commands using the `Commands` class
        available_commands = Extensions(agent_config).get_available_commands()

        # Save the existing command state to prevent duplication
        existing_command_states = {
            command["friendly_name"]: command["enabled"]
            for command in available_commands
        }

        for command in available_commands:
            command_friendly_name = command["friendly_name"]
            if command_friendly_name in existing_command_states:
                command_status = existing_command_states[command_friendly_name]
            else:
                continue  # Skip the command if it is not listed in the available commands

            toggle_status = st.checkbox(
                command_friendly_name,
                value=command_status,
                key=command_friendly_name,
            )
            command["enabled"] = toggle_status

        reduced_commands = {
            cmd["friendly_name"]: cmd["enabled"]
            for cmd in available_commands
            if cmd["friendly_name"] in existing_command_states
        }
        # Update the available commands back to the agent config
        agent.update_agent_config(reduced_commands, "commands")

    except Exception as e:
        st.error(f"Error loading agent configuration: {str(e)}")

if not new_agent:
    # Create a form for each button
    with st.form(key="update_agent_settings_form"):
        update_agent_settings_button = st.form_submit_button("Update Agent Settings")

    with st.form(key="wipe_memories_form"):
        wipe_memories_button = st.form_submit_button("Wipe Agent Memories")

    with st.form(key="delete_agent_form"):
        delete_agent_button = st.form_submit_button("Delete Agent")

    # Trigger actions on form submit
    if update_agent_settings_button:
        if agent_name:
            try:
                # Save commands in the desired format
                reduced_commands = {
                    cmd["friendly_name"]: cmd["enabled"] for cmd in available_commands
                }
                agent.update_agent_config(reduced_commands, "commands")
                agent.update_agent_config(agent_settings, "settings")
                st.success(f"Agent '{agent_name}' updated.")
            except Exception as e:
                st.error(f"Error updating agent: {str(e)}")

    if wipe_memories_button:
        if agent_name:
            try:
                agent.wipe_agent_memories(agent_name)
                st.success(f"Memories of agent '{agent_name}' wiped.")
            except Exception as e:
                st.error(f"Error wiping agent's memories: {str(e)}")

    if delete_agent_button:
        if agent_name:
            try:
                agent.delete_agent(agent_name)
                st.success(f"Agent '{agent_name}' deleted.")
                st.session_state.new_agent_name = ""  # Reset the selected agent
                st.experimental_rerun()  # Rerun the app to update the agent list
            except Exception as e:
                st.error(f"Error deleting agent: {str(e)}")
        else:
            st.error("Agent name is required.")



# Main.py
import streamlit as st
from components.agent_selector import agent_selector

st.set_page_config(
    page_title="AGiXT",
    page_icon=":robot:",
    layout="wide",
    initial_sidebar_state="expanded",
)
agent_name, agent = agent_selector()
st.markdown(
    """
    <img src="https://josh-xt.github.io/AGiXT/images/AGiXT.svg" width="100%">
    """,
    unsafe_allow_html=True,
)

st.markdown("## Useful Links")
st.markdown(
    """
- [AGiXT Documentation](https://josh-xt.github.io/AGiXT/)
- [AGiXT GitHub](https://github.com/Josh-XT/AGiXT)
- [AGiXT Discord](https://discord.gg/d3TkHRZcjD)"""
)



import streamlit as st
import auth_libs.Redirect as redir
import bcrypt
import logging
from auth_libs.Cfig import Cfig
from auth_libs.Users import load_users, save_user_data

CFIG = Cfig()
conf = CFIG.load_config()

# Check if the user is logged in
if st.session_state.get("logged_in"):
    # Redirect to the Profile page if so
    redir.nav_page("Profile")


def registration_form(auth):
    user_data = load_users()

    if auth and not conf["auth_setup_config"] == "No Login":
        st.write("Registration Form")
        email = st.text_input("Email")
        password = st.text_input("Password", type="password")
        confirm_password = st.text_input("Confirm Password", type="password")

        if st.button("Register"):
            if password == confirm_password:
                if email not in [user["email"] for user in user_data["users"]]:
                    hashed_password = bcrypt.hashpw(
                        password.encode("utf-8"), bcrypt.gensalt()
                    )
                    user_data["users"].append(
                        {
                            "email": email,
                            "password_hash": hashed_password.decode("utf-8"),
                        }
                    )
                    save_user_data(user_data)
                    st.success("Registration successful!")
                    st.experimental_rerun()  # Redirect to UI
                else:
                    st.error("Email already registered.")
            else:
                st.error("Passwords do not match.")

        st.stop()
    else:
        st.write("Registration Is Not Enabled!")


setup_CFIG = CFIG.load_config()
config = setup_CFIG["config"]
logging.info(config[0])
if config[0]:
    registration_form(auth=config[0])
else:
    st.write("Registration Is Not Enabled!")
